import os
import unittest

from conans.test.utils.cpp_test_files import cpp_hello_conan_files
from conans.test.utils.tools import TestClient, TestRequester, TestServer
from conans.util.files import save


class PutPropertiesTest(unittest.TestCase):

    def setUp(self):
        test_server = TestServer()
        self.servers = {"default": test_server}

    def create_empty_property_file_test(self):

        files = cpp_hello_conan_files("Hello0", "0.1", build=False)
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        client.save(files)
        client.run("export . lasote/stable")
        props_file = client.cache.put_headers_path
        self.assertTrue(os.path.exists(props_file))

    def put_properties_test(self):
        wanted_vars = {"MyHeader1": "MyHeaderValue1;MyHeaderValue2", "Other": "Value"}

        class RequesterCheckHeaders(TestRequester):
            def put(self, url, **kwargs):
                for name, value in wanted_vars.items():
                    value1 = kwargs["headers"][name]
                    if value1 != value:
                        raise Exception()
                return super(RequesterCheckHeaders, self).put(url, **kwargs)

        self.client = TestClient(requester_class=RequesterCheckHeaders, servers=self.servers,
                                 users={"default": [("lasote", "mypass")]})
        _create_property_files(self.client, wanted_vars)

        files = cpp_hello_conan_files("Hello0", "0.1", build=False)
        self.client.save(files)
        self.client.run("export . lasote/stable")
        self.client.run("upload Hello0/0.1@lasote/stable -c")


def _create_property_files(client, values):
    lines = ["#Some comment", " #Some comments"]
    for name, value in values.items():
        lines.append("%s=%s" % (name, value))
    save(client.cache.put_headers_path, "\n".join(lines))



import unittest

from conans.paths import CONANFILE, CONANFILE_TXT
from conans.test.utils.tools import TestClient


class TestPackageInfo(unittest.TestCase):

    def package_info_called_in_local_cache_test(self):
        client = TestClient()
        conanfile_tmp = '''
from conans import ConanFile
import os

class HelloConan(ConanFile):
    name = "%s"
    version = "1.0"
    build_policy = "missing"
    options = {"switch": ["1",  "0"]}
    default_options = "switch=0"
    %s
    
    def build(self):
        self.output.warn("Env var MYVAR={0}.".format(os.getenv("MYVAR", "")))

    def package_info(self):
        if self.options.switch == "0": 
            self.env_info.MYVAR = "foo"
        else:
            self.env_info.MYVAR = "bar"

'''
        for index in range(4):
            requires = "requires = 'Lib%s/1.0@conan/stable'" % index if index > 0 else ""
            conanfile = conanfile_tmp % ("Lib%s" % (index + 1), requires)
            client.save({CONANFILE: conanfile}, clean_first=True)
            client.run("create . conan/stable")

        txt = "[requires]\nLib4/1.0@conan/stable"
        client.save({CONANFILE_TXT: txt}, clean_first=True)
        client.run("install . -o *:switch=1")
        self.assertIn("Lib1/1.0@conan/stable: WARN: Env var MYVAR=.", client.out)
        self.assertIn("Lib2/1.0@conan/stable: WARN: Env var MYVAR=bar.", client.out)
        self.assertIn("Lib3/1.0@conan/stable: WARN: Env var MYVAR=bar.", client.out)
        self.assertIn("Lib4/1.0@conan/stable: WARN: Env var MYVAR=bar.", client.out)

        client.run("install . -o *:switch=0 --build Lib3")
        self.assertIn("Lib3/1.0@conan/stable: WARN: Env var MYVAR=foo", client.out)



import os
import time
import unittest

from nose.plugins.attrib import attr

from conans.model.ref import PackageReference
from conans.paths import CONANFILE
from conans.test.utils.cpp_test_files import cpp_hello_conan_files
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient
from conans.util.files import load


@attr("slow")
class ConanTestTest(unittest.TestCase):

    def test_partial_reference(self):
        # Create two packages to test with the same test
        conanfile = '''
from conans import ConanFile

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
'''
        client = TestClient()
        client.save({CONANFILE: conanfile})
        client.run("create . conan/stable")
        client.run("create . conan/testing")
        client.run("create . conan/foo")

        def test(conanfile_test, test_reference, path=None):
            path = path or "."
            client.save({os.path.join(path, CONANFILE): conanfile_test}, clean_first=True)
            client.run("test %s %s" % (path, test_reference))

        # Specify a valid name
        test('''
from conans import ConanFile

class HelloTestConan(ConanFile):
    requires = ["Hello/0.1@conan/stable", ]
    def test(self):
        self.output.warn("Tested ok!")
''', "Hello/0.1@conan/stable")
        self.assertIn("Tested ok!", client.out)

        # Specify a complete reference but not matching with the requires, it's ok, the
        # require could be a tool or whatever
        test('''
from conans import ConanFile

class HelloTestConan(ConanFile):
    requires = "Hello/0.1@conan/stable"
    def test(self):
        self.output.warn("Tested ok!")
''', "Hello/0.1@conan/foo")
        self.assertIn("Tested ok!", client.out)

    def test_package_env_test(self):
        client = TestClient()
        conanfile = '''
from conans import ConanFile

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    def package_info(self):
        self.env_info.PYTHONPATH.append("new/pythonpath/value")
        '''
        test_package = '''
import os
from conans import ConanFile

class HelloTestConan(ConanFile):
    requires = "Hello/0.1@lasote/testing"

    def build(self):
        assert("new/pythonpath/value" in os.environ["PYTHONPATH"])

    def test(self):
        assert("new/pythonpath/value" in os.environ["PYTHONPATH"])
'''

        client.save({"conanfile.py": conanfile, "test_package/conanfile.py": test_package})
        client.run("export . lasote/testing")
        client.run("test test_package Hello/0.1@lasote/testing --build missing")

    def fail_test_package_test(self):
        client = TestClient()
        conanfile = """
from conans import ConanFile

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    exports = "*"

    def package(self):
        self.copy("*")
"""
        test_conanfile = """
from conans import ConanFile, CMake
import os

class HelloReuseConan(ConanFile):
    requires = "Hello/0.1@lasote/stable"

    def test(self):
        pass
"""
        client.save({"conanfile.py": conanfile,
                     "FindXXX.cmake": "Hello FindCmake",
                     "test/conanfile.py": test_conanfile})
        client.run("create . lasote/stable")
        client.run("test test Hello/0.1@lasote/stable")
        pref = PackageReference.loads("Hello/0.1@lasote/stable:%s" % NO_SETTINGS_PACKAGE_ID)
        self.assertEqual("Hello FindCmake",
                         load(os.path.join(client.cache.package_layout(pref.ref).package(pref), "FindXXX.cmake")))
        client.save({"FindXXX.cmake": "Bye FindCmake"})
        client.run("test test Hello/0.1@lasote/stable")  # Test do not rebuild the package
        self.assertEqual("Hello FindCmake",
                         load(os.path.join(client.cache.package_layout(pref.ref).package(pref), "FindXXX.cmake")))
        client.run("create . lasote/stable")  # create rebuild the package
        self.assertEqual("Bye FindCmake",
                         load(os.path.join(client.cache.package_layout(pref.ref).package(pref), "FindXXX.cmake")))

    def conan_test_test(self):
        conanfile = '''
from conans import ConanFile, CMake
import os

class HelloReuseConan(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    requires = "Hello0/0.1@lasote/stable"
    generators = "cmake"

    def build(self):
        cmake = CMake(self)
        self.run('cmake "%s" %s' % (self.source_folder, cmake.command_line))
        self.run("cmake --build . %s" % cmake.build_config)

    def test(self):
        # equal to ./bin/greet, but portable win: .\bin\greet
        self.run(os.sep.join([".","bin", "greet"]))
        '''

        client = TestClient()
        files = cpp_hello_conan_files("Hello0", "0.1")
        print_build = 'self.output.warn("BUILD_TYPE=>%s" % self.settings.build_type)'
        files[CONANFILE] = files[CONANFILE].replace("def build(self):",
                                                    'def build(self):\n        %s' % print_build)

        # Add build_type setting
        files[CONANFILE] = files[CONANFILE].replace(', "arch"',
                                                    ', "arch", "build_type"')

        cmakelist = """set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
project(MyHello CXX)
cmake_minimum_required(VERSION 2.8)

include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
conan_basic_setup()

ADD_EXECUTABLE(greet main.cpp)
TARGET_LINK_LIBRARIES(greet ${CONAN_LIBS})
"""
        files["test_package/CMakeLists.txt"] = cmakelist
        files["test_package/conanfile.py"] = conanfile
        files["test_package/main.cpp"] = files["main.cpp"]
        client.save(files)
        client.run("create . lasote/stable -tf=None")
        time.sleep(1)  # Try to avoid windows errors in CI  (Cannot change permissions)
        client.run("test test_package Hello0/0.1@lasote/stable -s build_type=Release")
        self.assertIn('Hello Hello0', client.out)

        self.assertNotIn("WARN: conanbuildinfo.txt file not found", client.out)
        self.assertNotIn("WARN: conanenv.txt file not found", client.out)
        self.assertIn('Hello Hello0', client.out)
        client.run("test test_package Hello0/0.1@lasote/stable -s Hello0:build_type=Debug "
                   "-o Hello0:language=1 --build missing")
        self.assertIn('Hola Hello0', client.out)
        self.assertIn('BUILD_TYPE=>Debug', client.out)



import os
import unittest

from conans.client import tools
from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient
from conans.util.files import load, mkdir, rmdir

conanfile = '''
from conans import ConanFile
from conans.util.files import save, load
import os

class ConanFileToolsTest(ConanFile):
    name = "Pkg"
    version = "0.1"
    exports_sources = "*"
    generators = "cmake"

    def build(self):
        self.output.info("Source files: %s" % load(os.path.join(self.source_folder, "file.h")))
        save("myartifact.lib", "artifact contents!")
        save("subdir/myartifact2.lib", "artifact2 contents!")

    def package(self):
        self.copy("*.h")
        self.copy("*.lib")
'''


class DevInSourceFlowTest(unittest.TestCase):

    def _assert_pkg(self, folder):
        self.assertEqual(sorted(['file.h', 'myartifact.lib', 'subdir', 'conaninfo.txt',
                                 'conanmanifest.txt']),
                         sorted(os.listdir(folder)))
        self.assertEqual(load(os.path.join(folder, "myartifact.lib")),
                         "artifact contents!")
        self.assertEqual(load(os.path.join(folder, "subdir/myartifact2.lib")),
                         "artifact2 contents!")

    def parallel_folders_test(self):
        client = TestClient()
        repo_folder = os.path.join(client.current_folder, "recipe")
        build_folder = os.path.join(client.current_folder, "build")
        package_folder = os.path.join(client.current_folder, "pkg")
        mkdir(repo_folder)
        mkdir(build_folder)
        mkdir(package_folder)
        client.current_folder = repo_folder  # equivalent to git clone recipe
        client.save({"conanfile.py": conanfile,
                     "file.h": "file_h_contents!"})

        client.current_folder = build_folder
        client.run("install ../recipe")
        client.run("build ../recipe")
        client.current_folder = package_folder
        client.run("package ../recipe --build-folder=../build --package-folder='%s'" %
                   package_folder)
        self._assert_pkg(package_folder)
        client.current_folder = repo_folder
        client.run("export . lasote/testing")
        client.run("export-pkg . Pkg/0.1@lasote/testing -bf=../pkg")

        ref = ConanFileReference.loads("Pkg/0.1@lasote/testing")
        cache_package_folder = client.cache.package_layout(ref).packages()
        cache_package_folder = os.path.join(cache_package_folder,
                                            os.listdir(cache_package_folder)[0])
        self._assert_pkg(cache_package_folder)

    def insource_build_test(self):
        client = TestClient()
        repo_folder = client.current_folder
        package_folder = os.path.join(client.current_folder, "pkg")
        mkdir(package_folder)
        client.save({"conanfile.py": conanfile,
                     "file.h": "file_h_contents!"})

        client.run("install .")
        client.run("build .")
        client.current_folder = package_folder
        client.run("package .. --build-folder=.. --package-folder='%s' " % package_folder)
        self._assert_pkg(package_folder)
        client.current_folder = repo_folder
        client.run("export . lasote/testing")
        client.run("export-pkg . Pkg/0.1@lasote/testing -bf='%s' -if=." % package_folder)

        ref = ConanFileReference.loads("Pkg/0.1@lasote/testing")
        cache_package_folder = client.cache.package_layout(ref).packages()
        cache_package_folder = os.path.join(cache_package_folder,
                                            os.listdir(cache_package_folder)[0])
        self._assert_pkg(cache_package_folder)

    def child_build_test(self):
        client = TestClient()
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(build_folder)
        package_folder = os.path.join(build_folder, "package")
        mkdir(package_folder)
        client.save({"conanfile.py": conanfile,
                     "file.h": "file_h_contents!"})

        client.current_folder = build_folder
        client.run("install ..")
        client.run("build ..")
        client.current_folder = package_folder
        client.run("package ../.. --build-folder=../")
        self._assert_pkg(package_folder)
        rmdir(package_folder)  # IMPORTANT: Symptom that package + package_folder is not fitting
        # well now. (To discuss)
        # But I think now you choose you way to develop, local or cache, if you use conan export-pkg
        # you are done, if you use package() you need the "conan project" feature
        client.current_folder = build_folder
        client.run("export-pkg .. Pkg/0.1@lasote/testing --source-folder=.. ")

        ref = ConanFileReference.loads("Pkg/0.1@lasote/testing")
        cache_package_folder = client.cache.package_layout(ref).packages()
        cache_package_folder = os.path.join(cache_package_folder,
                                            os.listdir(cache_package_folder)[0])
        self._assert_pkg(cache_package_folder)


conanfile_out = '''
from conans import ConanFile
from conans.util.files import save, load
import os

class ConanFileToolsTest(ConanFile):
    name = "Pkg"
    version = "0.1"
    generators = "cmake"

    def source(self):
        save("file.h", "file_h_contents!")

    def build(self):
        self.output.info("Source files: %s" % load(os.path.join(self.source_folder, "file.h")))
        save("myartifact.lib", "artifact contents!")

    def package(self):
        self.copy("*.h")
        self.copy("*.lib")
'''


class DevOutSourceFlowTest(unittest.TestCase):

    def _assert_pkg(self, folder):
        self.assertEqual(sorted(['file.h', 'myartifact.lib', 'conaninfo.txt', 'conanmanifest.txt']),
                         sorted(os.listdir(folder)))

    def parallel_folders_test(self):
        client = TestClient()
        repo_folder = os.path.join(client.current_folder, "recipe")
        src_folder = os.path.join(client.current_folder, "src")
        build_folder = os.path.join(client.current_folder, "build")
        package_folder = os.path.join(build_folder, "package")
        mkdir(repo_folder)
        mkdir(src_folder)
        mkdir(build_folder)
        mkdir(package_folder)
        client.current_folder = repo_folder  # equivalent to git clone recipe
        client.save({"conanfile.py": conanfile_out})

        client.current_folder = build_folder
        client.run("install ../recipe")
        client.current_folder = src_folder
        client.run("install ../recipe")
        client.run("source ../recipe")
        client.current_folder = build_folder
        client.run("build ../recipe --source-folder=../src")
        client.current_folder = package_folder
        client.run("package ../../recipe --source-folder=../../src --build-folder=../")
        self._assert_pkg(package_folder)
        client.current_folder = repo_folder
        client.run("export . lasote/testing")
        client.run("export-pkg . Pkg/0.1@lasote/testing -bf=../build/package")

        ref = ConanFileReference.loads("Pkg/0.1@lasote/testing")
        cache_package_folder = client.cache.package_layout(ref).packages()
        cache_package_folder = os.path.join(cache_package_folder,
                                            os.listdir(cache_package_folder)[0])
        self._assert_pkg(cache_package_folder)

    def insource_build_test(self):
        client = TestClient()
        repo_folder = client.current_folder
        package_folder = os.path.join(client.current_folder, "pkg")
        mkdir(package_folder)
        client.save({"conanfile.py": conanfile_out})

        client.run("install .")
        client.run("source .")
        client.run("build . ")
        client.current_folder = package_folder
        client.run("package .. --build-folder=.. --package-folder='%s'" % package_folder)
        self._assert_pkg(package_folder)
        client.current_folder = repo_folder
        client.run("export . lasote/testing")
        client.run("export-pkg . Pkg/0.1@lasote/testing -bf=./pkg")

        ref = ConanFileReference.loads("Pkg/0.1@lasote/testing")
        cache_package_folder = client.cache.package_layout(ref).packages()
        cache_package_folder = os.path.join(cache_package_folder,
                                            os.listdir(cache_package_folder)[0])
        self._assert_pkg(cache_package_folder)

    def child_build_test(self):
        client = TestClient()
        repo_folder = client.current_folder
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(build_folder)
        package_folder = os.path.join(build_folder, "package")
        mkdir(package_folder)
        client.save({"conanfile.py": conanfile_out})

        client.current_folder = build_folder
        client.run("install ..")
        client.run("source ..")
        client.run("build .. --source-folder=.")
        client.current_folder = package_folder
        client.run("package ../.. --build-folder=../")
        self._assert_pkg(package_folder)
        rmdir(package_folder)
        client.current_folder = repo_folder

        client.run("export-pkg . Pkg/0.1@lasote/testing -bf=./build")

        ref = ConanFileReference.loads("Pkg/0.1@lasote/testing")
        cache_package_folder = client.cache.package_layout(ref).packages()
        cache_package_folder = os.path.join(cache_package_folder,
                                            os.listdir(cache_package_folder)[0])
        self._assert_pkg(cache_package_folder)

    def build_local_different_folders_test(self):
        # Real build, needed to ensure that the generator is put in the correct place and
        # cmake finds it, using an install_folder different from build_folder
        client = TestClient()
        client.run("new lib/1.0")
        # FIXME: this test, so it doesn't need to clone from github
        client.run("source . --source-folder src")

        # Patch the CMakeLists to include the generator file from a different folder
        install_dir = os.path.join(client.current_folder, "install_x86_64")
        tools.replace_in_file(os.path.join(client.current_folder, "src", "hello", "CMakeLists.txt"),
                              "${CMAKE_BINARY_DIR}/conanbuildinfo.cmake",
                              '"%s/conanbuildinfo.cmake"' % install_dir.replace("\\", "/"),
                              output=client.out)

        client.run("install . --install-folder install_x86_64 -s arch=x86_64")
        client.run("build . --build-folder build_x86_64 --install-folder '%s' "
                   "--source-folder src" % install_dir)
        self.assertTrue(os.path.exists(os.path.join(client.current_folder, "build_x86_64", "lib")))



import os
import unittest

from conans.paths import CONANFILE
from conans.test.utils.cpp_test_files import cpp_hello_conan_files
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient, TestServer
from conans.util.files import load, save


class InfoTest(unittest.TestCase):

    def setUp(self):
        test_server = TestServer(users={"lu": "mypass"})
        self.servers = {"default": test_server}
        self.clients = {}

        # Build a complex graph
        """
                            +-----------+
                   +------> |  H0       | <--------+
                   |        +------+----+          |
           private |               ^               |private
                   |               |               |
              +----+-----+    +----+------+   +----+------+
              |  H1a     |    | H1b       |   | H1c       |
              +----+-----+    +-----------+   +----+------+
                   ^                               ^
                   |                               |
                   |                               |
        +----------+-+                     +-------+------+
        |  H2a       | <------+    +-----> |   H2c        |
        +------------+        |    |       +--------------+
                              |    |
                          +---+----+---+
                          |  H3        |
                          +------------+

        """

        self._export("H0", "0.1")

        self._export("H1a", "0.1", deps=[("H0/0.1@lu/st", "private")])
        self._export("H1b", "0.1", deps=["H0/0.1@lu/st"])
        self._export("H1c", "0.1", deps=[("H0/0.1@lu/st", "private")])

        self._export("H2a", "0.1", deps=["H1a/0.1@lu/st"])
        self._export("H2c", "0.1", deps=["H1c/0.1@lu/st"])

        self._export("H3", "0.1", deps=["H2a/0.1@lu/st",
                                        "H2c/0.1@lu/st"])

    def _export(self, name=0, version=None, deps=None):
        client = TestClient(servers=self.servers, users={"default": [("lu", "mypass")]})
        self.clients[name] = client
        # Not necessary to actually build binaries
        files = cpp_hello_conan_files(name, version, deps, build=False)
        client.save(files, clean_first=True)
        client.run("export . lu/st")
        client.run("upload %s/%s@lu/st" % (name, version))

    def assert_last_line(self, client, line):
        lastline = str(client.out).splitlines()[-1]
        self.assertEqual(lastline, line)

    def info_build_test(self):
        """ Test that the output of 'conan info --build' is correct """
        # If we install H3 we need to build all except H1b
        self.clients["H3"].run("info . --build missing")
        self.assert_last_line(self.clients["H3"],
                              "H0/0.1@lu/st, H1a/0.1@lu/st, H1c/0.1@lu/st, "
                              "H2a/0.1@lu/st, H2c/0.1@lu/st")

        # If we install H0 we need to build nothing (current project)
        self.clients["H0"].run("info ./conanfile.py --build missing")
        self.assert_last_line(self.clients["H0"], "")

        # If we install H0 we need to build H0
        self.clients["H1a"].run("info conanfile.py --build missing")
        self.assert_last_line(self.clients["H1a"], "H0/0.1@lu/st")

        # If we build and upload H1a and H1c, no more H0 (private) is required
        self.clients["H3"].run("install H1a/0.1@lu/st --build ")
        self.clients["H3"].run("install H1c/0.1@lu/st --build ")
        self.clients["H3"].run("upload H1a/0.1@lu/st --all")
        self.clients["H3"].run("upload H1c/0.1@lu/st --all")

        self.clients["H3"].run("remove '*' -f")
        self.clients["H3"].run("info . --build missing")
        self.assert_last_line(self.clients["H3"],
                              "H2a/0.1@lu/st, H2c/0.1@lu/st")

        # But if we force to build all, all nodes have to be built
        self.clients["H3"].run("remove '*' -f")
        self.clients["H3"].run("info ./conanfile.py --build")
        self.assert_last_line(self.clients["H3"],
                              "H0/0.1@lu/st, H1a/0.1@lu/st, H1c/0.1@lu/st, "
                              "H2a/0.1@lu/st, H2c/0.1@lu/st")

        # Now upgrade the recipe H1a and upload it (but not the package)
        # so the package become outdated
        conanfile_path = os.path.join(self.clients["H1a"].current_folder, CONANFILE)
        conanfile = load(conanfile_path)
        conanfile += "\n# MODIFIED"
        save(conanfile_path, conanfile)
        self.clients["H1a"].run("export . lu/st")
        self.clients["H1a"].run("upload H1a/0.1@lu/st")  # NOW IS OUTDATED!

        # Without build outdated the built packages are the same
        self.clients["H3"].run("remove '*' -f")
        self.clients["H3"].run("info conanfile.py --build missing")

        if not self.clients["H3"].cache.config.revisions_enabled:
            self.assert_last_line(self.clients["H3"], "H2a/0.1@lu/st, H2c/0.1@lu/st")
        else:  # When revisions are enabled we just created a new one for H1a
            # when modifing the recipe so we need to rebuild it and its private H0
            self.assert_last_line(self.clients["H3"], 'H0/0.1@lu/st, H1a/0.1@lu/st, '
                                                      'H2a/0.1@lu/st, H2c/0.1@lu/st')

        # But with build outdated we have to build the private H0 (but only once) and H1a
        self.clients["H3"].run("remove '*' -f")
        self.clients["H3"].run("info . --build outdated")
        self.assert_last_line(self.clients["H3"],
                              "H0/0.1@lu/st, H1a/0.1@lu/st, H2a/0.1@lu/st, H2c/0.1@lu/st")

    def test_json_output_build(self):
        """ Test that the output of 'conan info --build' to json file is correct """
        json_file = os.path.join(temp_folder(), "output.json")
        json_output = '["H0/0.1@lu/st", "H1a/0.1@lu/st", "H1c/0.1@lu/st", ' \
                      '"H2a/0.1@lu/st", "H2c/0.1@lu/st"]'

        self.clients["H3"].run("info . --build missing --json=\"{}\"".format(json_file))
        self.assertEqual(load(json_file), json_output)

        self.clients["H3"].run("info . --build missing --json")
        self.assert_last_line(self.clients["H3"], json_output)






import unittest

from conans.model.package_metadata import PackageMetadata


class PackageMetadataTest(unittest.TestCase):

    def test_load_unload(self):
        a = PackageMetadata()
        a.recipe.revision = "rev"
        a.packages["ID"].recipe_revision = "rec_rev"
        a.packages["ID"].revision = "revp"
        a.packages["ID"].properties["Someprop"] = "23"

        tmp = a.dumps()

        b = PackageMetadata.loads(tmp)

        self.assertEqual(b, a)
        self.assertEqual(b.packages["ID"].properties["Someprop"], "23")

    def test_other_types_than_str(self):
        a = PackageMetadata()
        a.recipe.revision = "rev"
        a.packages["ID"].recipe_revision = 34
        a.packages["ID"].revision = {"23": 45}
        a.packages["ID"].properties["Someprop"] = [23, 2444]

        tmp = a.dumps()

        b = PackageMetadata.loads(tmp)

        self.assertEqual(b, a)
        self.assertEqual(b.packages["ID"].revision, {"23": 45})
        self.assertEqual(b.packages["ID"].properties["Someprop"], [23, 2444])



# coding=utf-8

import os
import stat
import unittest

import six

from conans.test.utils.test_files import temp_folder
from conans.util.files import remove, save


class RemoveTest(unittest.TestCase):

    def setUp(self):
        self.file = os.path.join(temp_folder(), 'file.txt')
        save(self.file, "some content")

    def test_remove(self):
        remove(self.file)
        self.assertFalse(os.path.exists(self.file))
        self.assertTrue(os.path.exists(os.path.dirname(self.file)))

    def test_remove_readonly(self):
        os.chmod(self.file, stat.S_IREAD|stat.S_IRGRP|stat.S_IROTH)
        with six.assertRaisesRegex(self, (IOError, OSError), "Permission denied"):
            save(self.file, "change the content")
        remove(self.file)
        self.assertFalse(os.path.exists(self.file))
        self.assertTrue(os.path.exists(os.path.dirname(self.file)))

    def test_remove_folder(self):
        dirname = os.path.dirname(self.file)
        self.assertRaises(AssertionError, remove, dirname)
        self.assertTrue(os.path.exists(dirname))




import os
import time
import unittest
from collections import OrderedDict
from time import sleep

from conans.model.ref import ConanFileReference, PackageReference
from conans.paths import CONAN_MANIFEST
from conans.test.utils.cpp_test_files import cpp_hello_conan_files
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient, TestServer, \
    TurboTestClient, GenConanfile
from conans.util.files import load, save


class InstallUpdateTest(unittest.TestCase):

    def setUp(self):
        test_server = TestServer()
        self.servers = {"myremote": test_server}
        self.client = TestClient(servers=self.servers, users={"myremote": [("lasote", "mypass")]})

    def update_binaries_test(self):
        conanfile = """from conans import ConanFile
from conans.tools import save
import os, random
class Pkg(ConanFile):
    def package(self):
        save(os.path.join(self.package_folder, "file.txt"), str(random.random()))
    def deploy(self):
        self.copy("file.txt")
"""
        self.client.save({"conanfile.py": conanfile})
        self.client.run("create . Pkg/0.1@lasote/testing")
        self.client.run("upload Pkg/0.1@lasote/testing --all -r=myremote")

        client2 = TestClient(servers=self.servers, users={"myremote": [("lasote", "mypass")]})
        client2.run("install Pkg/0.1@lasote/testing")
        value = load(os.path.join(client2.current_folder, "file.txt"))

        time.sleep(1)  # Make sure the new timestamp is later
        self.client.run("create . Pkg/0.1@lasote/testing")
        self.client.run("upload Pkg/0.1@lasote/testing --all")

        client2.run("install Pkg/0.1@lasote/testing")
        new_value = load(os.path.join(client2.current_folder, "file.txt"))
        self.assertEqual(value, new_value)

        client2.run("install Pkg/0.1@lasote/testing --update")
        self.assertIn("Current package is older than remote upstream one", client2.out)
        new_value = load(os.path.join(client2.current_folder, "file.txt"))
        self.assertNotEqual(value, new_value)

        # Now check newer local modifications are not overwritten
        time.sleep(1)  # Make sure the new timestamp is later
        self.client.run("create . Pkg/0.1@lasote/testing")
        self.client.run("upload Pkg/0.1@lasote/testing --all")

        client2.save({"conanfile.py": conanfile})
        client2.run("create . Pkg/0.1@lasote/testing")
        client2.run("install Pkg/0.1@lasote/testing")
        value2 = load(os.path.join(client2.current_folder, "file.txt"))
        client2.run("install Pkg/0.1@lasote/testing --update")
        self.assertIn("Current package is newer than remote upstream one", client2.out)
        new_value = load(os.path.join(client2.current_folder, "file.txt"))
        self.assertEqual(value2, new_value)

    def upload_doesnt_follow_pref_test(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    pass
"""
        servers = OrderedDict()
        servers['r1'] = TestServer()
        servers['r2'] = TestServer()
        client = TestClient(servers=servers, users={"r1": [("lasote", "mypass")],
                                                    "r2": [("lasote", "mypass")]})
        ref = "Pkg/0.1@lasote/testing"
        pref = "%s:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9" % ref
        client.save({"conanfile.py": conanfile})
        client.run("create . Pkg/0.1@lasote/testing")
        client.run("upload Pkg/0.1@lasote/testing --all -r r2")
        client.run("remote list_pref Pkg/0.1@lasote/testing")

        self.assertIn("%s: r2" % pref, client.out)
        client.run("remote remove_ref Pkg/0.1@lasote/testing")

        # It should upload both to r1 (default), not taking into account the pref to r2
        client.run("upload Pkg/0.1@lasote/testing --all")
        self.assertIn("Uploading package 1/1: %s to 'r1'" % NO_SETTINGS_PACKAGE_ID,
                      client.out)

    def install_update_following_pref_test(self):
        conanfile = """
import os, random
from conans import ConanFile, tools
class Pkg(ConanFile):
    def package(self):
        tools.save(os.path.join(self.package_folder, "file.txt"), str(random.random()))
"""
        servers = OrderedDict()
        servers["r1"] = TestServer()
        servers["r2"] = TestServer()
        client = TestClient(servers=servers, users={"r1": [("lasote", "mypass")],
                                                    "r2": [("lasote", "mypass")]})

        ref = "Pkg/0.1@lasote/testing"
        client.save({"conanfile.py": conanfile})
        client.run("create . %s" % ref)
        client.run("upload %s --all -r r2" % ref)
        client.run("upload %s --all -r r1" % ref)
        # Force recipe to follow r1
        client.run("remote update_ref %s r1" % ref)

        # Update package in r2 from a different client
        time.sleep(1)
        client2 = TestClient(servers=servers, users={"r1": [("lasote", "mypass")],
                                                     "r2": [("lasote", "mypass")]})
        ref = "Pkg/0.1@lasote/testing"
        client2.save({"conanfile.py": conanfile})
        client2.run("create . %s" % ref)
        client2.run("upload %s --all -r r2" % ref)

        # Update from client, it will get the binary from r2
        client.run("install %s --update" % ref)
        self.assertIn("Pkg/0.1@lasote/testing from 'r1' - Cache", client.out)
        self.assertIn("Retrieving package "
                      "%s from remote 'r2'" % NO_SETTINGS_PACKAGE_ID, client.out)

    def update_binaries_failed_test(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    pass
"""
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . Pkg/0.1@lasote/testing")
        client.run("install Pkg/0.1@lasote/testing --update")
        self.assertIn("Pkg/0.1@lasote/testing: WARN: Can't update, no remote defined",
                      client.out)

    def update_binaries_no_package_error_test(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    pass
"""
        client = TestClient(servers=self.servers, users={"myremote": [("lasote", "mypass")]})
        client.save({"conanfile.py": conanfile})
        client.run("create . Pkg/0.1@lasote/testing")
        client.run("upload Pkg/0.1@lasote/testing")
        client.run("remote add_pref Pkg/0.1@lasote/testing:"
                   "%s myremote" % NO_SETTINGS_PACKAGE_ID)
        client.run("install Pkg/0.1@lasote/testing --update")
        self.assertIn("Pkg/0.1@lasote/testing: WARN: Can't update, no package in remote",
                      client.out)

    def update_not_date_test(self):
        # Regression for https://github.com/conan-io/conan/issues/949
        files0 = cpp_hello_conan_files("Hello0", "1.0", build=False)
        files0["conanfile.py"] = files0["conanfile.py"].replace("settings = ", "# settings = ")
        self.client.save(files0)
        self.client.run("export . lasote/stable")
        files1 = cpp_hello_conan_files("Hello1", "1.0", build=False,
                                       deps=["Hello0/1.0@lasote/stable"])
        self.client.save(files1, clean_first=True)
        self.client.run("install . --build")
        self.client.run("upload Hello0/1.0@lasote/stable --all")

        self.client.run("remote list_ref")
        self.assertIn("Hello0/1.0@lasote/stable", self.client.out)
        self.client.run("remote list_pref Hello0/1.0@lasote/stable")
        reference = "Hello0/1.0@lasote/stable"
        package_reference = "%s:55a3af76272ead64e6f543c12ecece30f94d3eda" % reference
        self.assertIn(package_reference, self.client.out)

        ref = ConanFileReference.loads("Hello0/1.0@lasote/stable")
        pref = PackageReference(ref, "55a3af76272ead64e6f543c12ecece30f94d3eda")
        export_folder = self.client.cache.package_layout(ref).export()
        recipe_manifest = os.path.join(export_folder, CONAN_MANIFEST)
        package_folder = self.client.cache.package_layout(pref.ref).package(pref)
        package_manifest = os.path.join(package_folder, CONAN_MANIFEST)

        def timestamps():
            recipe_timestamp = load(recipe_manifest).splitlines()[0]
            package_timestamp = load(package_manifest).splitlines()[0]
            return recipe_timestamp, package_timestamp

        initial_timestamps = timestamps()

        time.sleep(1)

        # Change and rebuild package
        files0["helloHello0.h"] += " // useless comment"
        self.client.save(files0, clean_first=True)
        self.client.run("export . lasote/stable")
        self.client.run("install Hello0/1.0@lasote/stable --build")

        self.client.run("remote list_ref")
        # FIXME Conan 2.0 It should be a assertNotIn
        self.assertIn("Hello0/1.0@lasote/stable", self.client.out)

        self.client.run("remote list_pref Hello0/1.0@lasote/stable")
        # FIXME Conan 2.0 It should be a assertNotIn
        self.assertIn("Hello0/1.0@lasote/stable:55a3af76272ead64e6f543c12ecece30f94d3eda",
                      self.client.out)

        rebuild_timestamps = timestamps()
        self.assertNotEqual(rebuild_timestamps, initial_timestamps)

        # back to the consumer, try to update
        self.client.save(files1, clean_first=True)
        # First assign the preference to a remote, it has been cleared when exported locally
        self.client.run("install . --update")
        # *1 With revisions here is removing the package because it doesn't belong to the recipe

        self.assertIn("Hello0/1.0@lasote/stable from 'myremote' - Newer", self.client.out)
        failed_update_timestamps = timestamps()
        self.assertEqual(rebuild_timestamps, failed_update_timestamps)

        # hack manifests, put old time
        for manifest_file in (recipe_manifest, package_manifest):
            manifest = load(manifest_file)
            lines = manifest.splitlines()
            lines[0] = "123"
            save(manifest_file, "\n".join(lines))

        self.client.run("install . --update")
        update_timestamps = timestamps()
        self.assertEqual(update_timestamps, initial_timestamps)

    def reuse_test(self):
        files = cpp_hello_conan_files("Hello0", "1.0", build=False)

        self.client.save(files)
        self.client.run("export . lasote/stable")
        self.client.run("install Hello0/1.0@lasote/stable --build")
        self.client.run("upload Hello0/1.0@lasote/stable --all")

        client2 = TestClient(servers=self.servers, users={"myremote": [("lasote", "mypass")]})
        client2.run("install Hello0/1.0@lasote/stable")

        self.assertEqual(str(client2.out).count("Downloading conaninfo.txt"), 1)

        files["helloHello0.h"] = "//EMPTY!"
        self.client.save(files, clean_first=True)
        sleep(1)
        self.client.run("export . lasote/stable")
        self.client.run("install Hello0/1.0@lasote/stable --build")
        self.client.run("upload Hello0/1.0@lasote/stable --all")

        client2.run("install Hello0/1.0@lasote/stable --update")
        ref = ConanFileReference.loads("Hello0/1.0@lasote/stable")
        package_ids = client2.cache.package_layout(ref).conan_packages()
        pref = PackageReference(ref, package_ids[0])
        package_path = client2.cache.package_layout(ref).package(pref)
        header = load(os.path.join(package_path, "include/helloHello0.h"))
        self.assertEqual(header, "//EMPTY!")

    def remove_old_sources_test(self):
        # https://github.com/conan-io/conan/issues/1841
        test_server = TestServer()

        def upload(header_content):
            client = TestClient(servers={"default": test_server},
                                users={"default": [("lasote", "mypass")]})
            base = '''from conans import ConanFile
class ConanLib(ConanFile):
    exports_sources = "*"
    def package(self):
        self.copy("*")
'''
            client.save({"conanfile.py": base,
                         "header.h": header_content})
            client.run("create . Pkg/0.1@lasote/channel")
            client.run("upload * --confirm --all")
            return client

        client = upload("mycontent1")
        time.sleep(1)
        upload("mycontent2")

        client.run("install Pkg/0.1@lasote/channel -u")

        if client.cache.config.revisions_enabled:
            # The binary package is not updated but downloaded, because the local one we have
            # belongs to a different revision and it is removed
            self.assertIn("Pkg/0.1@lasote/channel:%s - Download" % NO_SETTINGS_PACKAGE_ID,
                          client.out)
        else:
            self.assertIn("Pkg/0.1@lasote/channel:%s - Update" % NO_SETTINGS_PACKAGE_ID,
                          client.out)
        self.assertIn("Pkg/0.1@lasote/channel: Retrieving package %s" % NO_SETTINGS_PACKAGE_ID,
                      client.out)
        ref = ConanFileReference.loads("Pkg/0.1@lasote/channel")
        pref = PackageReference(ref, NO_SETTINGS_PACKAGE_ID)
        header = os.path.join(client.cache.package_layout(pref.ref).package(pref), "header.h")
        self.assertEqual(load(header), "mycontent2")

    def fail_usefully_when_failing_retrieving_package_test(self):
        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        ref2 = ConanFileReference.loads("lib2/1.0@conan/stable")
        client = TurboTestClient(servers={"default": TestServer()})
        pref1 = client.create(ref)
        client.upload_all(ref)

        client.create(ref2, conanfile=GenConanfile().with_requirement(ref))
        client.upload_all(ref2)

        # remove only the package from pref1
        client.run("remove {} -p {} -f".format(pref1.ref, pref1.id))

        # Now fake the remote url to force a network failure
        client.run("remote update default http://this_not_exist8823.com")
        # Try to install ref2, it will try to download the binary for ref1
        client.run("install {}".format(ref2), assert_error=True)
        self.assertIn("ERROR: Error downloading binary package: '{}'".format(pref1), client.out)



import os
import subprocess

from six import string_types

from conans.client.tools.scm import Git, SVN
from conans.errors import ConanException
from conans.util.files import rmdir


def get_scm_data(conanfile):
    data = getattr(conanfile, "scm", None)
    if data is not None and isinstance(data, dict):
        return SCMData(conanfile)
    else:
        return None


def _get_dict_value(data, key, expected_type, default=None, disallowed_type=None):
    if key in data:
        r = data.get(key)
        if r is None:  # None is always a valid value
            return r
        if not isinstance(r, expected_type) or (disallowed_type and isinstance(r, disallowed_type)):
            type_str = "' or '".join([it.__name__ for it in expected_type]) \
                if isinstance(expected_type, tuple) else expected_type.__name__
            raise ConanException("SCM value for '{}' must be of type '{}'"
                                 " (found '{}')".format(key, type_str, type(r).__name__))
        return r
    return default


class SCMData(object):
    VERIFY_SSL_DEFAULT = True
    SHALLOW_DEFAULT = True

    def __init__(self, conanfile):
        data = getattr(conanfile, "scm")
        self.type = _get_dict_value(data, "type", string_types)
        self.url = _get_dict_value(data, "url", string_types)
        self.revision = _get_dict_value(data, "revision", string_types + (int, ),
                                        disallowed_type=bool)  # bool is subclass of integer
        self.verify_ssl = _get_dict_value(data, "verify_ssl", bool, SCMData.VERIFY_SSL_DEFAULT)
        self.username = _get_dict_value(data, "username", string_types)
        self.password = _get_dict_value(data, "password", string_types)
        self.subfolder = _get_dict_value(data, "subfolder", string_types)
        self.submodule = _get_dict_value(data, "submodule", string_types)
        self.shallow = _get_dict_value(data, "shallow", bool, SCMData.SHALLOW_DEFAULT)

    @property
    def capture_origin(self):
        return self.url == "auto"

    @property
    def capture_revision(self):
        return self.revision == "auto"

    @property
    def recipe_revision(self):
        if self.type in ["git", "svn"]:
            return self.revision
        raise ConanException("Not implemented recipe revision for %s" % self.type)

    def __repr__(self):
        d = {"url": self.url, "revision": self.revision, "username": self.username,
             "password": self.password, "type": self.type,
             "subfolder": self.subfolder, "submodule": self.submodule}
        if self.shallow != self.SHALLOW_DEFAULT:
            d.update({"shallow": self.shallow})
        if self.verify_ssl != self.VERIFY_SSL_DEFAULT:
            d.update({"verify_ssl": self.verify_ssl})
        d = {k: v for k, v in d.items() if v is not None}

        def _kv_to_string(key, value):
            if isinstance(value, bool):
                return '"{}": {}'.format(key, value)
            else:
                value_str = str(value).replace('"', r'\"')
                return '"{}": "{}"'.format(key, value_str)

        return '{' + ', '.join([_kv_to_string(k, v) for k, v in sorted(d.items())]) + '}'


class SCM(object):
    availables = {'git': Git, 'svn': SVN}

    def __init__(self, data, repo_folder, output):
        self._data = data
        self._output = output
        self.repo_folder = repo_folder
        # Finally instance a repo
        self.repo = self._get_repo()

    @classmethod
    def detect_scm(cls, folder):
        for name, candidate in cls.availables.items():
            try:
                candidate(folder).check_repo()
                return name
            except ConanException:
                pass
        return None

    def _get_repo(self):
        repo_class = self.availables.get(self._data.type)
        if not repo_class:
            raise ConanException("SCM not supported: %s" % self._data.type)

        return repo_class(folder=self.repo_folder, verify_ssl=self._data.verify_ssl,
                          username=self._data.username, password=self._data.password,
                          output=self._output)

    @property
    def excluded_files(self):
        return self.repo.excluded_files()

    def checkout(self):
        output = ""
        if self._data.type == "git":
            def use_not_shallow():
                out = self.repo.clone(url=self._data.url, shallow=False)
                out += self.repo.checkout(element=self._data.revision,
                                          submodule=self._data.submodule)
                return out

            def use_shallow():
                try:
                    out = self.repo.clone(url=self._data.url, branch=self._data.revision,
                                          shallow=True)
                except subprocess.CalledProcessError:
                    # remove the .git directory, otherwise, fallback clone cannot be successful
                    # it's completely safe to do here, as clone without branch expects
                    # empty directory
                    rmdir(os.path.join(self.repo_folder, ".git"))
                    out = use_not_shallow()
                else:
                    out += self.repo.checkout_submodules(submodule=self._data.submodule)
                return out

            if self._data.shallow:
                output += use_shallow()
            else:
                output += use_not_shallow()

        else:
            output += self.repo.checkout(url=self._data.url, revision=self._data.revision)
        return output

    def get_remote_url(self, remove_credentials):
        return self.repo.get_remote_url(remove_credentials=remove_credentials)

    def get_revision(self):
        return self.repo.get_revision()

    def is_pristine(self):
        return self.repo.is_pristine()

    def get_repo_root(self):
        return self.repo.get_repo_root()

    def get_qualified_remote_url(self, remove_credentials):
        if self._data.type == "git":
            return self.repo.get_remote_url(remove_credentials=remove_credentials)
        else:
            return self.repo.get_qualified_remote_url(remove_credentials=remove_credentials)

    def is_local_repository(self):
        return self.repo.is_local_repository()

    @staticmethod
    def clean_url(url):
        _, last_chunk = url.rsplit('/', 1)
        if '@' in last_chunk:  # Remove peg_revision
            url, peg_revision = url.rsplit('@', 1)
            return url
        return url

    def get_local_path_to_url(self, url):
        """ Compute the local path to the directory where the URL is pointing to (only make sense
            for CVS where chunks of the repository can be checked out isolated). The argument
            'url' should be contained inside the root url.
        """
        src_root = self.get_repo_root()

        if self._data.type == "git":
            return src_root

        url_root = SCM(self._data, src_root, self._output).get_remote_url(remove_credentials=True)
        if url_root:
            url = self.clean_url(url)
            src_path = os.path.join(src_root, os.path.relpath(url, url_root))
            return src_path



import fnmatch
import os
from collections import OrderedDict

from conans.client.generators.text import TXTGenerator
from conans.client.graph.build_mode import BuildMode
from conans.client.graph.graph import BINARY_BUILD, Node,\
    RECIPE_CONSUMER, RECIPE_VIRTUAL, BINARY_EDITABLE
from conans.client.graph.graph_binaries import GraphBinariesAnalyzer
from conans.client.graph.graph_builder import DepsGraphBuilder
from conans.errors import ConanException, conanfile_exception_formatter
from conans.model.conan_file import get_env_context_manager
from conans.model.graph_info import GraphInfo
from conans.model.graph_lock import GraphLock, GraphLockFile
from conans.model.ref import ConanFileReference
from conans.paths import BUILD_INFO
from conans.util.files import load


class _RecipeBuildRequires(OrderedDict):
    def __init__(self, conanfile):
        super(_RecipeBuildRequires, self).__init__()
        build_requires = getattr(conanfile, "build_requires", [])
        if not isinstance(build_requires, (list, tuple)):
            build_requires = [build_requires]
        for build_require in build_requires:
            self.add(build_require)

    def add(self, build_require):
        if not isinstance(build_require, ConanFileReference):
            build_require = ConanFileReference.loads(build_require)
        self[build_require.name] = build_require

    def __call__(self, build_require):
        self.add(build_require)

    def update(self, build_requires):
        for build_require in build_requires:
            self.add(build_require)

    def __str__(self):
        return ", ".join(str(r) for r in self.values())


class GraphManager(object):
    def __init__(self, output, cache, remote_manager, loader, proxy, resolver):
        self._proxy = proxy
        self._output = output
        self._resolver = resolver
        self._cache = cache
        self._remote_manager = remote_manager
        self._loader = loader

    def load_consumer_conanfile(self, conanfile_path, info_folder,
                                deps_info_required=False, test=None):
        """loads a conanfile for local flow: source, imports, package, build
        """
        try:
            graph_info = GraphInfo.load(info_folder)
            graph_lock_file = GraphLockFile.load(info_folder)
            graph_lock = graph_lock_file.graph_lock
            self._output.info("Using lockfile: '{}/conan.lock'".format(info_folder))
            profile = graph_lock_file.profile
            self._output.info("Using cached profile from lockfile")
        except IOError:  # Only if file is missing
            graph_lock = None
            # This is very dirty, should be removed for Conan 2.0 (source() method only)
            profile = self._cache.default_profile
            profile.process_settings(self._cache)
            name, version, user, channel = None, None, None, None
        else:
            name, version, user, channel, _ = graph_info.root
            profile.process_settings(self._cache, preprocess=False)
            # This is the hack of recovering the options from the graph_info
            profile.options.update(graph_info.options)
        processed_profile = profile
        if conanfile_path.endswith(".py"):
            lock_python_requires = None
            if graph_lock and not test:  # Only lock python requires if it is not test_package
                node_id = graph_lock.get_node(graph_info.root)
                lock_python_requires = graph_lock.python_requires(node_id)
            conanfile = self._loader.load_consumer(conanfile_path,
                                                   processed_profile=processed_profile, test=test,
                                                   name=name, version=version,
                                                   user=user, channel=channel,
                                                   lock_python_requires=lock_python_requires)
            with get_env_context_manager(conanfile, without_python=True):
                with conanfile_exception_formatter(str(conanfile), "config_options"):
                    conanfile.config_options()
                with conanfile_exception_formatter(str(conanfile), "configure"):
                    conanfile.configure()

                conanfile.settings.validate()  # All has to be ok!
                conanfile.options.validate()
        else:
            conanfile = self._loader.load_conanfile_txt(conanfile_path, processed_profile)

        load_deps_info(info_folder, conanfile, required=deps_info_required)

        return conanfile

    def load_graph(self, reference, create_reference, graph_info, build_mode, check_updates, update,
                   remotes, recorder, apply_build_requires=True):

        def _inject_require(conanfile, ref):
            """ test_package functionality requires injecting the tested package as requirement
            before running the install
            """
            require = conanfile.requires.get(ref.name)
            if require:
                require.ref = require.range_ref = ref
            else:
                conanfile.requires.add_ref(ref)
            conanfile._conan_user = ref.user
            conanfile._conan_channel = ref.channel

        # Computing the full dependency graph
        profile = graph_info.profile
        processed_profile = profile
        processed_profile.dev_reference = create_reference
        ref = None
        graph_lock = graph_info.graph_lock
        if isinstance(reference, list):  # Install workspace with multiple root nodes
            conanfile = self._loader.load_virtual(reference, processed_profile,
                                                  scope_options=False)
            root_node = Node(ref, conanfile, recipe=RECIPE_VIRTUAL)
        elif isinstance(reference, ConanFileReference):
            if not self._cache.config.revisions_enabled and reference.revision is not None:
                raise ConanException("Revisions not enabled in the client, specify a "
                                     "reference without revision")
            # create without test_package and install <ref>
            conanfile = self._loader.load_virtual([reference], processed_profile)
            root_node = Node(ref, conanfile, recipe=RECIPE_VIRTUAL)
            if graph_lock:  # Find the Node ID in the lock of current root
                graph_lock.find_consumer_node(root_node, reference)
        else:
            path = reference
            if path.endswith(".py"):
                test = str(create_reference) if create_reference else None
                lock_python_requires = None
                # do not try apply lock_python_requires for test_package/conanfile.py consumer
                if graph_lock and not create_reference:
                    if graph_info.root.name is None:
                        # If the graph_info information is not there, better get what we can from
                        # the conanfile
                        conanfile = self._loader.load_class(path)
                        graph_info.root = ConanFileReference(graph_info.root.name or conanfile.name,
                                                             graph_info.root.version or conanfile.version,
                                                             graph_info.root.user,
                                                             graph_info.root.channel, validate=False)
                    node_id = graph_lock.get_node(graph_info.root)
                    lock_python_requires = graph_lock.python_requires(node_id)

                conanfile = self._loader.load_consumer(path, processed_profile, test=test,
                                                       name=graph_info.root.name,
                                                       version=graph_info.root.version,
                                                       user=graph_info.root.user,
                                                       channel=graph_info.root.channel,
                                                       lock_python_requires=lock_python_requires)
                if create_reference:  # create with test_package
                    _inject_require(conanfile, create_reference)

                ref = ConanFileReference(conanfile.name, conanfile.version,
                                         conanfile._conan_user, conanfile._conan_channel,
                                         validate=False)
            else:
                conanfile = self._loader.load_conanfile_txt(path, processed_profile,
                                                            ref=graph_info.root)

            root_node = Node(ref, conanfile, recipe=RECIPE_CONSUMER)

            if graph_lock:  # Find the Node ID in the lock of current root
                graph_lock.find_consumer_node(root_node, create_reference)

        build_mode = BuildMode(build_mode, self._output)
        deps_graph = self._load_graph(root_node, check_updates, update,
                                      build_mode=build_mode, remotes=remotes,
                                      profile_build_requires=profile.build_requires,
                                      recorder=recorder,
                                      processed_profile=processed_profile,
                                      apply_build_requires=apply_build_requires,
                                      graph_lock=graph_lock)

        # THIS IS NECESSARY to store dependencies options in profile, for consumer
        # FIXME: This is a hack. Might dissapear if graph for local commands is always recomputed
        graph_info.options = root_node.conanfile.options.values
        if ref:
            graph_info.root = ref
        if graph_info.graph_lock is None:
            graph_info.graph_lock = GraphLock(deps_graph)
        else:
            graph_info.graph_lock.update_check_graph(deps_graph, self._output)

        version_ranges_output = self._resolver.output
        if version_ranges_output:
            self._output.success("Version ranges solved")
            for msg in version_ranges_output:
                self._output.info("    %s" % msg)
            self._output.writeln("")

        build_mode.report_matches()
        return deps_graph, conanfile

    @staticmethod
    def _get_recipe_build_requires(conanfile):
        conanfile.build_requires = _RecipeBuildRequires(conanfile)
        if hasattr(conanfile, "build_requirements"):
            with get_env_context_manager(conanfile):
                with conanfile_exception_formatter(str(conanfile), "build_requirements"):
                    conanfile.build_requirements()

        return conanfile.build_requires

    def _recurse_build_requires(self, graph, subgraph, builder, binaries_analyzer, check_updates,
                                update, build_mode, remotes, profile_build_requires, recorder,
                                processed_profile, graph_lock, apply_build_requires=True):
        """
        :param graph: This is the full dependency graph with all nodes from all recursions
        :param subgraph: A partial graph of the nodes that need to be evaluated and expanded
            at this recursion. Only the nodes belonging to this subgraph will get their package_id
            computed, and they will resolve build_requires if they need to be built from sources
        """

        binaries_analyzer.evaluate_graph(subgraph, build_mode, update, remotes)
        if not apply_build_requires:
            return

        for node in subgraph.ordered_iterate():
            # Virtual conanfiles doesn't have output, but conanfile.py and conanfile.txt do
            # FIXME: To be improved and build a explicit model for this
            if node.recipe == RECIPE_VIRTUAL:
                continue
            if (node.binary not in (BINARY_BUILD, BINARY_EDITABLE)
                    and node.recipe != RECIPE_CONSUMER):
                continue
            package_build_requires = self._get_recipe_build_requires(node.conanfile)
            str_ref = str(node.ref)
            new_profile_build_requires = []
            profile_build_requires = profile_build_requires or {}
            for pattern, build_requires in profile_build_requires.items():
                if ((node.recipe == RECIPE_CONSUMER and pattern == "&") or
                        (node.recipe != RECIPE_CONSUMER and pattern == "&!") or
                        fnmatch.fnmatch(str_ref, pattern)):
                            for build_require in build_requires:
                                if build_require.name in package_build_requires:  # Override defined
                                    # this is a way to have only one package Name for all versions
                                    # (no conflicts)
                                    # but the dict key is not used at all
                                    package_build_requires[build_require.name] = build_require
                                elif build_require.name != node.name:  # Profile one
                                    new_profile_build_requires.append(build_require)

            if package_build_requires:
                subgraph = builder.extend_build_requires(graph, node,
                                                         package_build_requires.values(),
                                                         check_updates, update, remotes,
                                                         processed_profile, graph_lock)
                self._recurse_build_requires(graph, subgraph, builder, binaries_analyzer,
                                             check_updates, update, build_mode,
                                             remotes, profile_build_requires, recorder,
                                             processed_profile, graph_lock)
                graph.nodes.update(subgraph.nodes)

            if new_profile_build_requires:
                subgraph = builder.extend_build_requires(graph, node, new_profile_build_requires,
                                                         check_updates, update, remotes,
                                                         processed_profile, graph_lock)
                self._recurse_build_requires(graph, subgraph, builder, binaries_analyzer,
                                             check_updates, update, build_mode,
                                             remotes, {}, recorder,
                                             processed_profile, graph_lock)
                graph.nodes.update(subgraph.nodes)

    def _load_graph(self, root_node, check_updates, update, build_mode, remotes,
                    profile_build_requires, recorder, processed_profile, apply_build_requires,
                    graph_lock):

        assert isinstance(build_mode, BuildMode)
        builder = DepsGraphBuilder(self._proxy, self._output, self._loader, self._resolver,
                                   recorder)
        graph = builder.load_graph(root_node, check_updates, update, remotes, processed_profile,
                                   graph_lock)
        binaries_analyzer = GraphBinariesAnalyzer(self._cache, self._output,
                                                  self._remote_manager)

        self._recurse_build_requires(graph, graph, builder, binaries_analyzer, check_updates, update,
                                     build_mode, remotes,
                                     profile_build_requires, recorder, processed_profile,
                                     graph_lock,
                                     apply_build_requires=apply_build_requires)

        # Sort of closures, for linking order
        inverse_levels = {n: i for i, level in enumerate(graph.inverse_levels()) for n in level}
        for node in graph.nodes:
            closure = node.public_closure
            closure.pop(node.name)
            node_order = list(closure.values())
            # List sort is stable, will keep the original order of closure, but prioritize levels
            node_order.sort(key=lambda n: inverse_levels[n])
            node.public_closure = node_order

        return graph


def load_deps_info(current_path, conanfile, required):

    def get_forbidden_access_object(field_name):
        class InfoObjectNotDefined(object):
            def __getitem__(self, item):
                raise ConanException("self.%s not defined. If you need it for a "
                                     "local command run 'conan install'" % field_name)
            __getattr__ = __getitem__

        return InfoObjectNotDefined()

    if not current_path:
        return
    info_file_path = os.path.join(current_path, BUILD_INFO)
    try:
        deps_cpp_info, deps_user_info, deps_env_info = TXTGenerator.loads(load(info_file_path))
        conanfile.deps_cpp_info = deps_cpp_info
        conanfile.deps_user_info = deps_user_info
        conanfile.deps_env_info = deps_env_info
    except IOError:
        if required:
            raise ConanException("%s file not found in %s\nIt is required for this command\n"
                                 "You can generate it using 'conan install'"
                                 % (BUILD_INFO, current_path))
        conanfile.deps_cpp_info = get_forbidden_access_object("deps_cpp_info")
        conanfile.deps_user_info = get_forbidden_access_object("deps_user_info")
    except ConanException:
        raise ConanException("Parse error in '%s' file in %s" % (BUILD_INFO, current_path))



# coding=utf-8

import os
import textwrap
import unittest

from parameterized import parameterized

from conans.model.editable_layout import DEFAULT_LAYOUT_FILE, LAYOUTS_FOLDER
from conans.test.utils.tools import TestClient
from conans.util.files import save
from conans.test.utils.test_files import temp_folder


class HeaderOnlyLibTestClient(TestClient):
    header = textwrap.dedent("""\
        #include <iostream>

        void hello() {{
            std::cout << "Hello {word}!" << std::endl;
            std::cout << "...using {origin}" << std::endl;
        }}
        """)

    conanfile = textwrap.dedent("""\
        import os
        from conans import ConanFile, tools

        class Pkg(ConanFile):
            name = "MyLib"
            version = "0.1"

            exports_sources = "*"

            def package(self):
                self.copy("*.hpp", dst="include-inrepo", src="src/include-inrepo")
                self.copy("*.hpp", dst="include-cache", src="src/include-cache")

            def package_info(self):
                self.cpp_info.libs = ["MyLib", "otra", ]
                self.cpp_info.defines = ["MyLibDEFINES",]
                self.cpp_info.libdirs = ["MyLib-libdirs", ]
                self.cpp_info.includedirs = ["src/include-local", ]
        """)

    conan_inrepo_layout = textwrap.dedent("""\
        [includedirs]
        src/include-inrepo
        """)

    conan_cache_layout = textwrap.dedent("""\
        [MyLib/0.1@user/editable:includedirs]
        src/include-cache
        """)

    def __init__(self, use_repo_file, use_cache_file, *args, **kwargs):
        super(HeaderOnlyLibTestClient, self).__init__(*args, **kwargs)

        self.save({"conanfile.py": self.conanfile,
                   "src/include-inrepo/hello.hpp": self.header.format(word="EDITABLE",
                                                                      origin="inrepo"),
                   "src/include-cache/hello.hpp": self.header.format(word="EDITABLE",
                                                                     origin="cache"),
                   "src/include-local/hello.hpp": self.header.format(word="EDITABLE",
                                                                     origin="local")
                   })

        if use_repo_file:
            self.save({"mylayout": self.conan_inrepo_layout, })

        if use_cache_file:
            file_path = os.path.join(self.cache.cache_folder, LAYOUTS_FOLDER, DEFAULT_LAYOUT_FILE)
            save(file_path, self.conan_cache_layout)

    def update_hello_word(self, hello_word):
        self.save({"src/include-inrepo/hello.hpp": self.header.format(word=hello_word,
                                                                      origin='inrepo'),
                   "src/include-cache/hello.hpp": self.header.format(word=hello_word,
                                                                     origin='cache'),
                   "src/include-local/hello.hpp": self.header.format(word=hello_word,
                                                                     origin='local')})


class EditableReferenceTest(unittest.TestCase):

    @parameterized.expand([(False, True), (True, False), (True, True), (False, False)])
    def test_header_only(self, use_repo_file, use_cache_file):
        # We need two clients sharing the same Conan cache
        cache_folder = temp_folder()

        # Editable project
        client_editable = HeaderOnlyLibTestClient(use_repo_file=use_repo_file,
                                                  use_cache_file=use_cache_file,
                                                  cache_folder=cache_folder)
        if use_repo_file:
            client_editable.run("editable add . MyLib/0.1@user/editable --layout=mylayout")
        else:
            client_editable.run("editable add . MyLib/0.1@user/editable")

        # Consumer project
        client = TestClient(cache_folder=cache_folder)
        conanfile_py = """
import os
from conans import ConanFile, CMake

class TestConan(ConanFile):
    name = "pkg"
    version = "0.0"

    requires = "MyLib/0.1@user/editable"
    settings = "os", "compiler", "build_type", "arch"
    generators = "cmake"
    exports_sources = "src/*"

    def build(self):
        cmake = CMake(self)
        cmake.configure(source_folder="src")
        cmake.build()

        os.chdir("bin")
        self.run(".%shello" % os.sep)

"""
        cmakelists = """
cmake_minimum_required(VERSION 2.8.12)
project(MyHello CXX)

include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
conan_basic_setup()

add_executable(hello main.cpp)
"""
        main_cpp = """
#include "hello.hpp"

int main() {
    hello();
}
"""

        client.save({"conanfile.py": conanfile_py,
                     "src/CMakeLists.txt": cmakelists,
                     "src/main.cpp": main_cpp})

        # Build consumer project
        client.run("create . pkg/0.0@user/testing")
        self.assertIn("    MyLib/0.1@user/editable from user folder - Editable", client.out)
        self.assertIn("    MyLib/0.1@user/editable:"
                      "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Editable", client.out)
        self.assertIn("Hello EDITABLE!", client.out)
        if use_repo_file:  # Repo file will override folders from cache
            self.assertIn("...using inrepo", client.out)
            self.assertNotIn("...using cache", client.out)
            self.assertNotIn("...using local", client.out)
        elif use_cache_file:
            self.assertIn("...using cache", client.out)
            self.assertNotIn("...using inrepo", client.out)
            self.assertNotIn("...using local", client.out)
        else:
            self.assertIn("...using local", client.out)
            self.assertNotIn("...using inrepo", client.out)
            self.assertNotIn("...using cache", client.out)

        # Modify editable and build again
        client_editable.update_hello_word(hello_word="EDITED")
        client.run("create . pkg/0.0@user/testing")
        self.assertIn("Hello EDITED!", client.out)
        if use_repo_file:  # Repo file will override folders from cache
            self.assertIn("...using inrepo", client.out)
            self.assertNotIn("...using cache", client.out)
            self.assertNotIn("...using local", client.out)
        elif use_cache_file:
            self.assertIn("...using cache", client.out)
            self.assertNotIn("...using inrepo", client.out)
            self.assertNotIn("...using local", client.out)
        else:
            self.assertIn("...using local", client.out)
            self.assertNotIn("...using inrepo", client.out)
            self.assertNotIn("...using cache", client.out)



import os
import unittest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient, TestServer

conanfile_basic = """
from conans import ConanFile

class AConan(ConanFile):
    name = "basic"
    version = "0.1"
    
    def package_info(self):
        self.cpp_info.defines = ["ACONAN"]
"""

complete_hook = """
from conans.model.ref import ConanFileReference


def pre_export(output, conanfile, conanfile_path, reference, **kwargs):
    assert conanfile
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())

def post_export(output, conanfile, conanfile_path, reference, **kwargs):
    assert conanfile
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())

def pre_source(output, conanfile, conanfile_path, **kwargs):
    assert conanfile
    output.info("conanfile_path=%s" % conanfile_path)
    if conanfile.in_local_cache:
        output.info("reference=%s" % kwargs["reference"].full_str())

def post_source(output, conanfile, conanfile_path, **kwargs):
    assert conanfile
    output.info("conanfile_path=%s" % conanfile_path)
    if conanfile.in_local_cache:
        output.info("reference=%s" % kwargs["reference"].full_str())

def pre_build(output, conanfile, **kwargs):
    assert conanfile
    if conanfile.in_local_cache:
        output.info("reference=%s" % kwargs["reference"].full_str())
        output.info("package_id=%s" % kwargs["package_id"])
    else:
        output.info("conanfile_path=%s" % kwargs["conanfile_path"])

def post_build(output, conanfile, **kwargs):
    assert conanfile
    if conanfile.in_local_cache:
        output.info("reference=%s" % kwargs["reference"].full_str())
        output.info("package_id=%s" % kwargs["package_id"])
    else:
        output.info("conanfile_path=%s" % kwargs["conanfile_path"])

def pre_package(output, conanfile, conanfile_path, **kwargs):
    assert conanfile
    output.info("conanfile_path=%s" % conanfile_path)
    if conanfile.in_local_cache:
        output.info("reference=%s" % kwargs["reference"].full_str())
        output.info("package_id=%s" % kwargs["package_id"])

def post_package(output, conanfile, conanfile_path, **kwargs):
    assert conanfile
    output.info("conanfile_path=%s" % conanfile_path)
    if conanfile.in_local_cache:
        output.info("reference=%s" % kwargs["reference"].full_str())
        output.info("package_id=%s" % kwargs["package_id"])

def pre_upload(output, conanfile_path, reference, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("remote.name=%s" % remote.name)

def post_upload(output, conanfile_path, reference, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("remote.name=%s" % remote.name)

def pre_upload_recipe(output, conanfile_path, reference, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("remote.name=%s" % remote.name)

def post_upload_recipe(output, conanfile_path, reference, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("remote.name=%s" % remote.name)

def pre_upload_package(output, conanfile_path, reference, package_id, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("package_id=%s" % package_id)
    output.info("remote.name=%s" % remote.name)

def post_upload_package(output, conanfile_path, reference, package_id, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("package_id=%s" % package_id)
    output.info("remote.name=%s" % remote.name)

def pre_download(output, reference, remote, **kwargs):
    output.info("reference=%s" % reference.full_str())
    output.info("remote.name=%s" % remote.name)

def post_download(output, conanfile_path, reference, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("remote.name=%s" % remote.name)

def pre_download_recipe(output, reference, remote, **kwargs):
    output.info("reference=%s" % reference.full_str())
    output.info("remote.name=%s" % remote.name)

def post_download_recipe(output, conanfile_path, reference, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("remote.name=%s" % remote.name)

def pre_download_package(output, conanfile_path, reference, package_id, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("package_id=%s" % package_id)
    output.info("remote.name=%s" % remote.name)

def post_download_package(output, conanfile_path, reference, package_id, remote, **kwargs):
    output.info("conanfile_path=%s" % conanfile_path)
    output.info("reference=%s" % reference.full_str())
    output.info("package_id=%s" % package_id)
    output.info("remote.name=%s" % remote.name)
    
def pre_package_info(output, conanfile, reference, **kwargs):
    output.info("reference=%s" % reference.full_str())
    output.info("conanfile.cpp_info.defines=%s" % conanfile.cpp_info.defines)

def post_package_info(output, conanfile, reference, **kwargs):
    output.info("reference=%s" % reference.full_str())
    output.info("conanfile.cpp_info.defines=%s" % conanfile.cpp_info.defines)
"""

HEADER = "[HOOK - complete_hook] {method_name}():"
REFERENCE_LOCAL = "basic/0.1@PROJECT"
REFERENCE_CACHE = "basic/0.1@danimtb/testing"
PACKAGE_ID = NO_SETTINGS_PACKAGE_ID

custom_module = """
def my_printer(output):
    output.info("my_printer(): CUSTOM MODULE")
"""

my_hook = """
from custom_module.custom import my_printer

def pre_export(output, conanfile_path, reference, **kwargs):
    my_printer(output)
"""


class HookTest(unittest.TestCase):

    def default_hook_test(self):
        client = TestClient()
        self.assertTrue(client.cache.hooks_path.endswith("hooks"))
        client.save({"conanfile.py": conanfile_basic})
        client.run("export . danimtb/testing")
        self.assertIn("[HOOK - attribute_checker.py] pre_export(): "
                      "WARN: Conanfile doesn't have 'url'", client.out)
        self.assertIn("[HOOK - attribute_checker.py] pre_export(): "
                      "WARN: Conanfile doesn't have 'description'", client.out)
        self.assertIn("[HOOK - attribute_checker.py] pre_export(): "
                      "WARN: Conanfile doesn't have 'license'", client.out)

    def complete_hook_test(self):
        server = TestServer([], users={"danimtb": "pass"})
        client = TestClient(servers={"default": server}, users={"default": [("danimtb", "pass")]})
        hook_path = os.path.join(client.cache.hooks_path, "complete_hook",
                                 "complete_hook.py")
        client.save({hook_path: complete_hook, "conanfile.py": conanfile_basic})
        conanfile_path = os.path.join(client.current_folder, "conanfile.py")
        conanfile_cache_path = client.cache.package_layout(
            ConanFileReference("basic", "0.1", "danimtb", "testing")).conanfile()
        client.run("config set hooks.complete_hook/complete_hook.py")

        client.run("source .")
        self._check_source(conanfile_path, client.out)

        client.run("install .")
        client.run("build .")
        self._check_build(conanfile_path, client.out)

        client.run("package .")
        self._check_package(conanfile_path, client.out)

        client.run("export . danimtb/testing")
        self._check_export(conanfile_path, conanfile_cache_path, client.out)

        client.run("export-pkg . danimtb/testing")
        self._check_export(conanfile_path, conanfile_cache_path, client.out)
        self._check_export_pkg(conanfile_cache_path, client.out)

        client.run("remove * --force")
        client.run('export-pkg . danimtb/testing -pf . ')
        self._check_export(conanfile_path, conanfile_cache_path, client.out)
        self._check_export_pkg(conanfile_cache_path, client.out)

        client.run("remove * --force")
        client.run("create . danimtb/testing")
        self._check_export(conanfile_path, conanfile_cache_path, client.out)  # Export gets
        self._check_source(conanfile_cache_path, client.out, in_cache=True)
        self._check_build(conanfile_cache_path, client.out, in_cache=True)
        self._check_package(conanfile_cache_path, client.out, in_cache=True)
        self._check_package_info(client.out)

        client.run("upload basic/0.1@danimtb/testing -r default")
        self._check_upload(conanfile_cache_path, client.out)
        self._check_upload_recipe(conanfile_cache_path, client.out)
        client.run("upload basic/0.1@danimtb/testing -r default --all")
        self._check_upload(conanfile_cache_path, client.out)
        self._check_upload_recipe(conanfile_cache_path, client.out)
        self._check_upload_package(conanfile_cache_path, client.out)

        client.run("remove * --force")
        client.run("download basic/0.1@danimtb/testing --recipe")
        self._check_download(conanfile_cache_path, client.out)
        self._check_download_recipe(conanfile_cache_path, client.out)
        client.run("remove * --force")
        client.run("download basic/0.1@danimtb/testing")
        self._check_download(conanfile_cache_path, client.out)
        self._check_download_recipe(conanfile_cache_path, client.out)
        self._check_download_package(conanfile_cache_path, client.out)

        client.run("remove * --force")
        client.run("install basic/0.1@danimtb/testing")
        self._check_download_recipe(conanfile_cache_path, client.out)
        self._check_download_package(conanfile_cache_path, client.out)
        self._check_package_info(client.out)

    def _check_source(self, conanfile_path, out, in_cache=False):
        reference = REFERENCE_CACHE if in_cache else REFERENCE_LOCAL
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_source(): "
                      "conanfile_path=%s" % conanfile_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_source(): "
                      "conanfile_path=%s" % conanfile_path, out)
        if in_cache:
            self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_source(): "
                          "reference=%s" % reference, out)
            self.assertIn("[HOOK - complete_hook/complete_hook.py] post_source(): "
                          "reference=%s" % reference, out)

    def _check_build(self, conanfile_path,  out, in_cache=False):
        reference = REFERENCE_CACHE if in_cache else REFERENCE_LOCAL
        if in_cache:
            self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_build(): "
                          "reference=%s" % reference, out)
            self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_build(): "
                          "package_id=%s" % PACKAGE_ID, out)
            self.assertIn("[HOOK - complete_hook/complete_hook.py] post_build(): "
                          "reference=%s" % reference, out)
            self.assertIn("[HOOK - complete_hook/complete_hook.py] post_build(): "
                          "package_id=%s" % PACKAGE_ID, out)
        else:
            self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_build(): "
                          "conanfile_path=%s" % conanfile_path, out)
            self.assertIn("[HOOK - complete_hook/complete_hook.py] post_build(): "
                          "conanfile_path=%s" % conanfile_path, out)

    def _check_package(self, conanfile_path, out, in_cache=False):
        reference = REFERENCE_CACHE if in_cache else REFERENCE_LOCAL
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_package(): "
                      "conanfile_path=%s" % conanfile_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_package(): "
                      "conanfile_path=%s" % conanfile_path, out)
        if in_cache:
            self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_package(): "
                          "reference=%s" % reference, out)
            self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_package(): "
                          "package_id=%s" % PACKAGE_ID, out)
            self.assertIn("[HOOK - complete_hook/complete_hook.py] post_package(): "
                          "reference=%s" % reference, out)
            self.assertIn("[HOOK - complete_hook/complete_hook.py] post_package(): "
                          "package_id=%s" % PACKAGE_ID, out)

    def _check_export(self, conanfile_path, conanfile_cache_path, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_export(): "
                      "conanfile_path=%s" % conanfile_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_export(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_export(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_export(): "
                      "reference=%s" % REFERENCE_CACHE, out)

    def _check_export_pkg(self, conanfile_cache_path, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_package(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_package(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_package(): "
                      "package_id=%s" % PACKAGE_ID, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_package(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_package(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_package(): "
                      "package_id=%s" % PACKAGE_ID, out)

    def _check_upload(self, conanfile_cache_path, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload(): "
                      "remote.name=default", out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload(): "
                      "remote.name=default", out)

    def _check_upload_recipe(self, conanfile_cache_path, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload_recipe(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload_recipe(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload_recipe(): "
                      "remote.name=default", out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload_recipe(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload_recipe(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload_recipe(): "
                      "remote.name=default", out)

    def _check_upload_package(self, conanfile_cache_path, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload_package(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload_package(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload_package(): "
                      "package_id=%s" % PACKAGE_ID, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_upload_package(): "
                      "remote.name=default", out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload_package(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload_package(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload_package(): "
                      "package_id=%s" % PACKAGE_ID, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_upload_package(): "
                      "remote.name=default", out)

    def _check_download(self, conanfile_cache_path, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_download(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_download(): "
                      "remote.name=default", out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download(): "
                      "remote.name=default", out)

    def _check_download_recipe(self, conanfile_cache_path, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_download_recipe(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_download_recipe(): "
                      "remote.name=default", out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download_recipe(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download_recipe(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download_recipe(): "
                      "remote.name=default", out)

    def _check_download_package(self, conanfile_cache_path, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_download_package(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_download_package(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_download_package(): "
                      "package_id=%s" % PACKAGE_ID, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_download_package(): "
                      "remote.name=default", out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download_package(): "
                      "conanfile_path=%s" % conanfile_cache_path, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download_package(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download_package(): "
                      "package_id=%s" % PACKAGE_ID, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_download_package(): "
                      "remote.name=default", out)

    def _check_package_info(self, out):
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_package_info(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] pre_package_info(): "
                      "conanfile.cpp_info.defines=[]", out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_package_info(): "
                      "reference=%s" % REFERENCE_CACHE, out)
        self.assertIn("[HOOK - complete_hook/complete_hook.py] post_package_info(): "
                      "conanfile.cpp_info.defines=['ACONAN']", out)

    def import_hook_test(self):
        client = TestClient()
        hook_path = os.path.join(client.cache.hooks_path, "my_hook", "my_hook.py")
        init_path = os.path.join(client.cache.hooks_path, "my_hook", "custom_module",
                                 "__init__.py")
        custom_path = os.path.join(client.cache.hooks_path, "my_hook", "custom_module",
                                   "custom.py")
        client.save({init_path: "",
                     custom_path: custom_module,
                     hook_path: my_hook,
                     "conanfile.py": conanfile_basic})
        client.run("config set hooks.my_hook/my_hook.py")
        client.run("export . danimtb/testing")
        self.assertIn("[HOOK - my_hook/my_hook.py] pre_export(): my_printer(): CUSTOM MODULE",
                      client.out)



import os
import unittest
from collections import OrderedDict

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient, TestServer, NO_SETTINGS_PACKAGE_ID, TurboTestClient
from conans.util.files import load


class DownloadTest(unittest.TestCase):

    def download_recipe_test(self):
        server = TestServer()
        servers = {"default": server}
        client = TurboTestClient(servers=servers, users={"default": [("lasote", "mypass")]})

        # Test download of the recipe only
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "pkg"
    version = "0.1"
"""
        ref = ConanFileReference.loads("pkg/0.1@lasote/stable")
        client.create(ref, conanfile)
        client.upload_all(ref)
        client.remove_all()

        client.run("download pkg/0.1@lasote/stable --recipe")

        self.assertIn("Downloading conanfile.py", client.out)
        self.assertNotIn("Downloading conan_package.tgz", client.out)
        export = client.cache.package_layout(ref).export()
        self.assertTrue(os.path.exists(os.path.join(export, "conanfile.py")))
        self.assertEqual(conanfile, load(os.path.join(export, "conanfile.py")))
        conan = client.cache.package_layout(ref).base_folder()
        self.assertFalse(os.path.exists(os.path.join(conan, "package")))

    def download_with_sources_test(self):
        server = TestServer()
        servers = OrderedDict()
        servers["default"] = server
        servers["other"] = TestServer()

        client = TestClient(servers=servers, users={"default": [("lasote", "mypass")],
                                                    "other": [("lasote", "mypass")]})
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "pkg"
    version = "0.1"
    exports_sources = "*"
"""
        client.save({"conanfile.py": conanfile,
                     "file.h": "myfile.h",
                     "otherfile.cpp": "C++code"})
        client.run("export . lasote/stable")

        ref = ConanFileReference.loads("pkg/0.1@lasote/stable")
        client.run("upload pkg/0.1@lasote/stable")
        client.run("remove pkg/0.1@lasote/stable -f")

        client.run("download pkg/0.1@lasote/stable")
        self.assertIn("Downloading conan_sources.tgz", client.out)
        source = client.cache.package_layout(ref).export_sources()
        self.assertEqual("myfile.h", load(os.path.join(source, "file.h")))
        self.assertEqual("C++code", load(os.path.join(source, "otherfile.cpp")))

    def download_reference_without_packages_test(self):
        server = TestServer()
        servers = {"default": server}

        client = TestClient(servers=servers, users={"default": [("lasote", "mypass")]})
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "pkg"
    version = "0.1"
"""
        client.save({"conanfile.py": conanfile})
        client.run("export . lasote/stable")

        ref = ConanFileReference.loads("pkg/0.1@lasote/stable")

        client.run("upload pkg/0.1@lasote/stable")
        client.run("remove pkg/0.1@lasote/stable -f")

        client.run("download pkg/0.1@lasote/stable")
        # Check 'No remote binary packages found' warning
        self.assertIn("WARN: No remote binary packages found in remote", client.out)
        # Check at least conanfile.py is downloaded
        self.assertTrue(os.path.exists(client.cache.package_layout(ref).conanfile()))

    def download_reference_with_packages_test(self):
        server = TestServer()
        servers = {"default": server}

        client = TurboTestClient(servers=servers, users={"default": [("lasote", "mypass")]})
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "pkg"
    version = "0.1"
    settings = "os"
"""
        ref = ConanFileReference.loads("pkg/0.1@lasote/stable")

        client.create(ref, conanfile)
        client.upload_all(ref)
        client.remove_all()

        client.run("download pkg/0.1@lasote/stable")

        package_layout = client.cache.package_layout(ref)

        package_folder = os.path.join(package_layout.packages(),
                                      os.listdir(package_layout.packages())[0])
        # Check not 'No remote binary packages found' warning
        self.assertNotIn("WARN: No remote binary packages found in remote", client.out)
        # Check at conanfile.py is downloaded
        self.assertTrue(os.path.exists(package_layout.conanfile()))
        # Check package folder created
        self.assertTrue(os.path.exists(package_folder))

    def test_download_wrong_id(self):
        client = TurboTestClient(servers={"default": TestServer()},
                                 users={"default": [("lasote", "mypass")]})

        ref = ConanFileReference.loads("pkg/0.1@lasote/stable")
        client.export(ref)
        client.upload_all(ref)
        client.remove_all()

        client.run("download pkg/0.1@lasote/stable:wrong", assert_error=True)
        self.assertIn("ERROR: Binary package not found: 'pkg/0.1@lasote/stable:wrong'",
                      client.out)

    def test_download_pattern(self):
        client = TestClient()
        client.run("download pkg/*@user/channel", assert_error=True)
        self.assertIn("Provide a valid full reference without wildcards", client.out)

    def download_full_reference_test(self):
        server = TestServer()
        servers = {"default": server}

        client = TurboTestClient(servers=servers, users={"default": [("lasote", "mypass")]})

        ref = ConanFileReference.loads("pkg/0.1@lasote/stable")
        client.create(ref)
        client.upload_all(ref)
        client.remove_all()

        client.run("download pkg/0.1@lasote/stable:{}".format(NO_SETTINGS_PACKAGE_ID))

        package_layout = client.cache.package_layout(ref)
        package_folder = os.path.join(package_layout.packages(),
                                      os.listdir(package_layout.packages())[0])
        # Check not 'No remote binary packages found' warning
        self.assertNotIn("WARN: No remote binary packages found in remote", client.out)
        # Check at conanfile.py is downloaded
        self.assertTrue(os.path.exists(package_layout.conanfile()))
        # Check package folder created
        self.assertTrue(os.path.exists(package_folder))

    def test_download_with_full_reference_and_p(self):
        client = TestClient()
        client.run("download pkg/0.1@user/channel:{package_id} -p {package_id}".
                   format(package_id="dupqipa4tog2ju3pncpnrzbim1fgd09g"),
                   assert_error=True)
        self.assertIn("Use a full package reference (preferred) or the `--package`"
                      " command argument, but not both.", client.out)

    def test_download_with_package_and_recipe_args(self):
        client = TestClient();
        client.run("download eigen/3.3.4@conan/stable --recipe --package fake_id",
                   assert_error=True)

        self.assertIn("ERROR: recipe parameter cannot be used together with package", client.out)

    def download_package_argument_test(self):
        server = TestServer()
        servers = {"default": server}

        client = TurboTestClient(servers=servers, users={"default": [("lasote", "mypass")]})

        ref = ConanFileReference.loads("pkg/0.1@lasote/stable")
        client.create(ref)
        client.upload_all(ref)
        client.remove_all()

        client.run("download pkg/0.1@lasote/stable -p {}".format(NO_SETTINGS_PACKAGE_ID))

        package_layout = client.cache.package_layout(ref)
        package_folder = os.path.join(package_layout.packages(),
                                      os.listdir(package_layout.packages())[0])
        # Check not 'No remote binary packages found' warning
        self.assertNotIn("WARN: No remote binary packages found in remote", client.out)
        # Check at conanfile.py is downloaded
        self.assertTrue(os.path.exists(package_layout.conanfile()))
        # Check package folder created
        self.assertTrue(os.path.exists(package_folder))

    def download_not_found_reference_test(self):
        server = TestServer()
        servers = {"default": server}
        client = TurboTestClient(servers=servers, users={"default": [("lasote", "mypass")]})
        client.run("download pkg/0.1@lasote/stable", assert_error=True)
        self.assertIn("ERROR: Recipe not found: 'pkg/0.1@lasote/stable'", client.out)



import os
import shutil

import six

from conans.client import tools
from conans.client.cmd.export import export_recipe, export_source
from conans.errors import ConanException, ConanExceptionInUserConanfileMethod, \
    conanfile_exception_formatter
from conans.model.conan_file import get_env_context_manager
from conans.model.scm import SCM, get_scm_data
from conans.paths import CONANFILE, CONAN_MANIFEST, EXPORT_SOURCES_TGZ_NAME, EXPORT_TGZ_NAME
from conans.util.files import (set_dirty, is_dirty, load, mkdir, rmdir, set_dirty_context_manager,
                               walk)


def complete_recipe_sources(remote_manager, cache, conanfile, ref, remotes):
    """ the "exports_sources" sources are not retrieved unless necessary to build. In some
    occassions, conan needs to get them too, like if uploading to a server, to keep the recipes
    complete
    """
    sources_folder = cache.package_layout(ref, conanfile.short_paths).export_sources()
    if os.path.exists(sources_folder):
        return None

    if conanfile.exports_sources is None:
        mkdir(sources_folder)
        return None

    # If not path to sources exists, we have a problem, at least an empty folder
    # should be there
    current_remote = cache.package_layout(ref).load_metadata().recipe.remote
    if current_remote:
        current_remote = remotes[current_remote]
    if not current_remote:
        raise ConanException("Error while trying to get recipe sources for %s. "
                             "No remote defined" % str(ref))

    export_path = cache.package_layout(ref).export()
    remote_manager.get_recipe_sources(ref, export_path, sources_folder, current_remote)


def merge_directories(src, dst, excluded=None):
    src = os.path.normpath(src)
    dst = os.path.normpath(dst)
    excluded = excluded or []
    excluded = [os.path.normpath(entry) for entry in excluded]

    def is_excluded(origin_path):
        if origin_path == dst:
            return True
        rel_path = os.path.normpath(os.path.relpath(origin_path, src))
        if rel_path in excluded:
            return True
        return False

    def link_to_rel(pointer_src):
        linkto = os.readlink(pointer_src)
        if not os.path.isabs(linkto):
            linkto = os.path.join(os.path.dirname(pointer_src), linkto)

        # Check if it is outside the sources
        out_of_source = os.path.relpath(linkto, os.path.realpath(src)).startswith(".")
        if out_of_source:
            # May warn about out of sources symlink
            return

        # Create the symlink
        linkto_rel = os.path.relpath(linkto, os.path.dirname(pointer_src))
        pointer_dst = os.path.normpath(os.path.join(dst, os.path.relpath(pointer_src, src)))
        os.symlink(linkto_rel, pointer_dst)

    for src_dir, dirs, files in walk(src, followlinks=True):
        if is_excluded(src_dir):
            dirs[:] = []
            continue

        if os.path.islink(src_dir):
            link_to_rel(src_dir)
            dirs[:] = []  # Do not enter subdirectories
            continue

        # Overwriting the dirs will prevents walk to get into them
        files[:] = [d for d in files if not is_excluded(os.path.join(src_dir, d))]

        dst_dir = os.path.normpath(os.path.join(dst, os.path.relpath(src_dir, src)))
        if not os.path.exists(dst_dir):
            os.makedirs(dst_dir)
        for file_ in files:
            src_file = os.path.join(src_dir, file_)
            dst_file = os.path.join(dst_dir, file_)
            if os.path.islink(src_file):
                link_to_rel(src_file)
            else:
                shutil.copy2(src_file, dst_file)


def config_source_local(src_folder, conanfile, conanfile_path, hook_manager):
    """ Entry point for the "conan source" command.
    """
    conanfile_folder = os.path.dirname(conanfile_path)

    def get_sources_from_exports():
        if conanfile_folder != src_folder:
            conanfile.output.info("Executing exports to: %s" % src_folder)
            export_recipe(conanfile, conanfile_folder, src_folder)
            export_source(conanfile, conanfile_folder, src_folder)

    _run_source(conanfile, conanfile_path, src_folder, hook_manager, reference=None, cache=None,
                local_sources_path=conanfile_folder,
                get_sources_from_exports=get_sources_from_exports)


def config_source(export_folder, export_source_folder, src_folder, conanfile, output,
                  conanfile_path, reference, hook_manager, cache):
    """ Implements the sources configuration when a package is going to be built in the
    local cache.
    """

    def remove_source(raise_error=True):
        output.warn("This can take a while for big packages")
        try:
            rmdir(src_folder)
        except BaseException as e_rm:
            set_dirty(src_folder)
            msg = str(e_rm)
            if six.PY2:
                msg = str(e_rm).decode("latin1")  # Windows prints some chars in latin1
            output.error("Unable to remove source folder %s\n%s" % (src_folder, msg))
            output.warn("**** Please delete it manually ****")
            if raise_error or isinstance(e_rm, KeyboardInterrupt):
                raise ConanException("Unable to remove source folder")

    sources_pointer = cache.package_layout(reference).scm_folder()
    local_sources_path = load(sources_pointer) if os.path.exists(sources_pointer) else None
    if is_dirty(src_folder):
        output.warn("Trying to remove corrupted source folder")
        remove_source()
    elif conanfile.build_policy_always:
        output.warn("Detected build_policy 'always', trying to remove source folder")
        remove_source()
    elif local_sources_path and os.path.exists(local_sources_path):
        output.warn("Detected 'scm' auto in conanfile, trying to remove source folder")
        remove_source()

    if not os.path.exists(src_folder):  # No source folder, need to get it
        with set_dirty_context_manager(src_folder):
            mkdir(src_folder)

            def get_sources_from_exports():
                # so self exported files have precedence over python_requires ones
                merge_directories(export_folder, src_folder)
                # Now move the export-sources to the right location
                merge_directories(export_source_folder, src_folder)

            _run_source(conanfile, conanfile_path, src_folder, hook_manager, reference,
                        cache, local_sources_path,
                        get_sources_from_exports=get_sources_from_exports)


def _run_source(conanfile, conanfile_path, src_folder, hook_manager, reference, cache,
                local_sources_path, get_sources_from_exports):
    """Execute the source core functionality, both for local cache and user space, in order:
        - Calling pre_source hook
        - Getting sources from SCM
        - Getting sources from exported folders in the local cache
        - Clean potential TGZ and other files in the local cache
        - Executing the recipe source() method
        - Calling post_source hook
    """
    conanfile.source_folder = src_folder
    conanfile.build_folder = None
    conanfile.package_folder = None
    with tools.chdir(src_folder):
        try:
            with get_env_context_manager(conanfile):
                hook_manager.execute("pre_source", conanfile=conanfile,
                                     conanfile_path=conanfile_path,
                                     reference=reference)
                output = conanfile.output
                output.info('Configuring sources in %s' % src_folder)
                _run_scm(conanfile, src_folder, local_sources_path, output, cache=cache)

                get_sources_from_exports()

                if cache:
                    _clean_source_folder(src_folder)  # TODO: Why is it needed in cache?
                with conanfile_exception_formatter(conanfile.display_name, "source"):
                    conanfile.source()

                hook_manager.execute("post_source", conanfile=conanfile,
                                     conanfile_path=conanfile_path,
                                     reference=reference)
        except ConanExceptionInUserConanfileMethod:
            raise
        except Exception as e:
            raise ConanException(e)


def _clean_source_folder(folder):
    for f in (EXPORT_TGZ_NAME, EXPORT_SOURCES_TGZ_NAME, CONANFILE+"c",
              CONANFILE+"o", CONANFILE, CONAN_MANIFEST):
        try:
            os.remove(os.path.join(folder, f))
        except OSError:
            pass
    try:
        shutil.rmtree(os.path.join(folder, "__pycache__"))
    except OSError:
        pass


def _run_scm(conanfile, src_folder, local_sources_path, output, cache):
    scm_data = get_scm_data(conanfile)
    if not scm_data:
        return

    dest_dir = os.path.normpath(os.path.join(src_folder, scm_data.subfolder or ""))
    if cache:
        # When in cache, capturing the sources from user space is done only if exists
        if not local_sources_path or not os.path.exists(local_sources_path):
            local_sources_path = None
    else:
        # In user space, if revision="auto", then copy
        if scm_data.capture_origin or scm_data.capture_revision:  # FIXME: or clause?
            scm = SCM(scm_data, local_sources_path, output)
            scm_url = scm_data.url if scm_data.url != "auto" else \
                scm.get_qualified_remote_url(remove_credentials=True)

            src_path = scm.get_local_path_to_url(url=scm_url)
            if src_path:
                local_sources_path = src_path
        else:
            local_sources_path = None

    if local_sources_path and conanfile.develop:
        excluded = SCM(scm_data, local_sources_path, output).excluded_files
        output.info("Getting sources from folder: %s" % local_sources_path)
        merge_directories(local_sources_path, dest_dir, excluded=excluded)
    else:
        output.info("Getting sources from url: '%s'" % scm_data.url)
        scm = SCM(scm_data, dest_dir, output)
        scm.checkout()

    if cache:
        # This is a bit weird. Why after a SCM should we remove files. Maybe check conan 2.0
        _clean_source_folder(dest_dir)  # TODO: Why removing in the cache? There is no danger.



import os
import unittest

from conans.client.store.localdb import LocalDB
from conans.test.utils.test_files import temp_folder


class LocalStoreTest(unittest.TestCase):

    def localdb_test(self):
        tmp_dir = temp_folder()
        db_file = os.path.join(tmp_dir, "dbfile")
        localdb = LocalDB.create(db_file)

        # Test write and read login
        user, token = localdb.get_login("myurl1")
        self.assertIsNone(user)
        self.assertIsNone(token)

        localdb.set_login(("pepe", "token"), "myurl1")
        user, token = localdb.get_login("myurl1")
        self.assertEqual("pepe", user)
        self.assertEqual("token", token)
        self.assertEqual("pepe", localdb.get_username("myurl1"))



import warnings

from conans.client.build.cppstd_flags import cppstd_flag
from conans.errors import ConanException
from conans.util.log import logger


def preprocess(settings):
    _fill_runtime(settings)
    _check_cppstd(settings)


def _check_cppstd(settings):
    compiler = settings.get_safe("compiler")
    compiler_version = settings.get_safe("compiler.version")
    cppstd = settings.get_safe("cppstd")
    compiler_cppstd = settings.get_safe("compiler.cppstd")

    if not cppstd and not compiler_cppstd:
        return

    # Checks: one or the other, but not both
    if cppstd and compiler_cppstd:
        raise ConanException("Do not use settings 'compiler.cppstd' together with 'cppstd'."
                             " Use only the former one.")

    if cppstd:
        warnings.warn("Setting 'cppstd' is deprecated in favor of 'compiler.cppstd'")

    if compiler not in ("gcc", "clang", "apple-clang", "Visual Studio"):
        return

    # Check that we have a flag available for that value of the C++ Standard
    def check_flag_available(values_range, value, setting_id):
        available = [v for v in values_range if cppstd_flag(compiler, compiler_version, v)]
        if str(value) not in available:
            raise ConanException("The specified '%s=%s' is not available "
                                 "for '%s %s'. Possible values are %s'" % (setting_id,
                                                                           value,
                                                                           compiler,
                                                                           compiler_version,
                                                                           available))

    if cppstd:
        check_flag_available(settings.cppstd.values_range, cppstd, "cppstd")
    else:
        check_flag_available(settings.compiler.cppstd.values_range,
                             compiler_cppstd, "compiler.cppstd")


def _fill_runtime(settings):
    try:
        if settings.compiler == "Visual Studio":
            if settings.get_safe("compiler.runtime") is None:
                settings.compiler.runtime = "MDd" if settings.get_safe("build_type") == "Debug" \
                                                  else "MD"
                logger.info("Setting 'compiler.runtime' not declared, automatically "
                            "adjusted to '%s'" % settings.compiler.runtime)
    except Exception:  # If the settings structure doesn't match these general
        # asumptions, like unexistant runtime
        pass



import os
import six
import sys
from colorama import Fore, Style

from conans.util.env_reader import get_env
from conans.util.files import decode_text


def colorama_initialize():
    # Respect color env setting or check tty if unset
    color_set = "CONAN_COLOR_DISPLAY" in os.environ
    if ((color_set and get_env("CONAN_COLOR_DISPLAY", 1))
            or (not color_set
                and hasattr(sys.stdout, "isatty")
                and sys.stdout.isatty())):
        import colorama
        if get_env("PYCHARM_HOSTED"):  # in PyCharm disable convert/strip
            colorama.init(convert=False, strip=False)
        else:
            colorama.init()
        color = True
    else:
        color = False
    return color


class Color(object):
    """ Wrapper around colorama colors that are undefined in importing
    """
    RED = Fore.RED  # @UndefinedVariable
    WHITE = Fore.WHITE  # @UndefinedVariable
    CYAN = Fore.CYAN  # @UndefinedVariable
    GREEN = Fore.GREEN  # @UndefinedVariable
    MAGENTA = Fore.MAGENTA  # @UndefinedVariable
    BLUE = Fore.BLUE  # @UndefinedVariable
    YELLOW = Fore.YELLOW  # @UndefinedVariable
    BLACK = Fore.BLACK  # @UndefinedVariable

    BRIGHT_RED = Style.BRIGHT + Fore.RED  # @UndefinedVariable
    BRIGHT_BLUE = Style.BRIGHT + Fore.BLUE  # @UndefinedVariable
    BRIGHT_YELLOW = Style.BRIGHT + Fore.YELLOW  # @UndefinedVariable
    BRIGHT_GREEN = Style.BRIGHT + Fore.GREEN  # @UndefinedVariable
    BRIGHT_CYAN = Style.BRIGHT + Fore.CYAN   # @UndefinedVariable
    BRIGHT_WHITE = Style.BRIGHT + Fore.WHITE   # @UndefinedVariable
    BRIGHT_MAGENTA = Style.BRIGHT + Fore.MAGENTA   # @UndefinedVariable


if get_env("CONAN_COLOR_DARK", 0):
    Color.WHITE = Fore.BLACK
    Color.CYAN = Fore.BLUE
    Color.YELLOW = Fore.MAGENTA
    Color.BRIGHT_WHITE = Fore.BLACK
    Color.BRIGHT_CYAN = Fore.BLUE
    Color.BRIGHT_YELLOW = Fore.MAGENTA
    Color.BRIGHT_GREEN = Fore.GREEN


class ConanOutput(object):
    """ wraps an output stream, so it can be pretty colored,
    and auxiliary info, success, warn methods for convenience.
    """

    def __init__(self, stream, stream_err=None, color=False):
        self._stream = stream
        self._stream_err = stream_err or stream
        self._color = color

    @property
    def is_terminal(self):
        return hasattr(self._stream, "isatty") and self._stream.isatty()

    def writeln(self, data, front=None, back=None, error=False):
        self.write(data, front, back, newline=True, error=error)

    def write(self, data, front=None, back=None, newline=False, error=False):
        if six.PY2:
            if isinstance(data, str):
                data = decode_text(data)  # Keep python 2 compatibility

        if self._color and (front or back):
            data = "%s%s%s%s" % (front or '', back or '', data, Style.RESET_ALL)
        if newline:
            data = "%s\n" % data

        # https://github.com/conan-io/conan/issues/4277
        # Windows output locks produce IOErrors
        for _ in range(3):
            try:
                if error:
                    self._stream_err.write(data)
                else:
                    self._stream.write(data)
                break
            except IOError:
                import time
                time.sleep(0.02)
            except UnicodeError:
                data = data.encode("utf8").decode("ascii", "ignore")

        self._stream.flush()

    def info(self, data):
        self.writeln(data, Color.BRIGHT_CYAN)

    def highlight(self, data):
        self.writeln(data, Color.BRIGHT_MAGENTA)

    def success(self, data):
        self.writeln(data, Color.BRIGHT_GREEN)

    def warn(self, data):
        self.writeln("WARN: {}".format(data), Color.BRIGHT_YELLOW, error=True)

    def error(self, data):
        self.writeln("ERROR: {}".format(data), Color.BRIGHT_RED, error=True)

    def input_text(self, data):
        self.write(data, Color.GREEN)

    def rewrite_line(self, line):
        tmp_color = self._color
        self._color = False
        TOTAL_SIZE = 70
        LIMIT_SIZE = 32  # Hard coded instead of TOTAL_SIZE/2-3 that fails in Py3 float division
        if len(line) > TOTAL_SIZE:
            line = line[0:LIMIT_SIZE] + " ... " + line[-LIMIT_SIZE:]
        self.write("\r%s%s" % (line, " " * (TOTAL_SIZE - len(line))))
        self._stream.flush()
        self._color = tmp_color

    def flush(self):
        self._stream.flush()


class ScopedOutput(ConanOutput):
    def __init__(self, scope, output):
        self.scope = scope
        self._stream = output._stream
        self._stream_err = output._stream_err
        self._color = output._color

    def write(self, data, front=None, back=None, newline=False, error=False):
        assert self.scope != "virtual", "printing with scope==virtual"
        super(ScopedOutput, self).write("%s: " % self.scope, front=front, back=back,
                                        newline=False, error=error)
        super(ScopedOutput, self).write("%s" % data, front=Color.BRIGHT_WHITE, back=back,
                                        newline=newline, error=error)



import unittest


from conans.test.utils.tools import TestClient, TestServer
from collections import namedtuple


class Error200NoJson(unittest.TestCase):

    def test_error_no_json(self):
        class RequesterMock(object):
            def __init__(self, *args, **kwargs):
                pass

            def get(self, *args, **kwargs):  # @UnusedVariable
                # Response must be binary, it is decoded in RestClientCommon
                return namedtuple("Response", "status_code headers content ok")(200, {}, b'<>',
                                                                                True)

        # https://github.com/conan-io/conan/issues/3432
        client = TestClient(servers={"default": TestServer()},
                            requester_class=RequesterMock,
                            users={"default": [("lasote", "mypass")]})

        client.run("install pkg/ref@user/testing", assert_error=True)
        self.assertIn("ERROR: <>", client.out)
        self.assertIn("Response from remote is not json, but 'None'", client.out)

    def test_error_broken_json(self):
        class RequesterMock(object):
            def __init__(self, *args, **kwargs):
                pass

            def get(self, *args, **kwargs):  # @UnusedVariable
                # Response must be binary, it is decoded in RestClientCommon
                headers = {"Content-Type": "application/json"}
                return namedtuple("Response", "status_code headers content ok")(200, headers,
                                                                                b'<>', True)

        # https://github.com/conan-io/conan/issues/3432
        client = TestClient(servers={"default": TestServer()},
                            requester_class=RequesterMock,
                            users={"default": [("lasote", "mypass")]})

        client.run("install pkg/ref@user/testing", assert_error=True)
        self.assertIn("Remote responded with broken json: <>", client.out)

    def test_error_json(self):
        class RequesterMock(object):
            def __init__(self, *args, **kwargs):
                pass

            def get(self, *args, **kwargs):  # @UnusedVariable
                # Response must be binary, it is decoded in RestClientCommon
                headers = {"Content-Type": "application/json"}
                return namedtuple("Response", "status_code headers content ok")(200, headers,
                                                                                b'[1, 2, 3]', True)

        # https://github.com/conan-io/conan/issues/3432
        client = TestClient(servers={"default": TestServer()},
                            requester_class=RequesterMock,
                            users={"default": [("lasote", "mypass")]})

        client.run("install pkg/ref@user/testing", assert_error=True)
        self.assertIn("ERROR: Unexpected server response [1, 2, 3]", client.out)




from collections import OrderedDict
from collections import namedtuple

import six
from parameterized import parameterized

from conans.errors import ConanException
from conans.model.ref import ConanFileReference
from conans.model.requires import Requirements
from conans.test.unittests.model.transitive_reqs_test import GraphTest
from conans.test.utils.conanfile import TestConanFile
from conans.test.utils.tools import GenConanfile, TurboTestClient, TestServer, \
    NO_SETTINGS_PACKAGE_ID
from conans.test.utils.tools import test_processed_profile


def _get_nodes(graph, name):
    """ return all the nodes matching a particular name. Could be >1 in case
    that private requirements embed different versions
    """
    return [n for n in graph.nodes if n.conanfile.name == name]


Edge = namedtuple("Edge", "src dst")


def _get_edges(graph):
    edges = set()
    for n in graph.nodes:
        edges.update([Edge(n, neigh) for neigh in n.neighbors()])
    return edges


def _clear_revs(requires):
    for require in requires.values():
        require.ref = require.ref.copy_clear_rev()
    return requires


class VersionRangesTest(GraphTest):

    def setUp(self):
        super(VersionRangesTest, self).setUp()

        for v in ["0.1", "0.2", "0.3", "1.1", "1.1.2", "1.2.1", "2.1", "2.2.1"]:
            say_content = TestConanFile("Say", v)
            say_ref = ConanFileReference.loads("Say/%s@myuser/testing" % v)
            self.retriever.save_recipe(say_ref, say_content)

    def build_graph(self, content, update=False):
        self.loader.cached_conanfiles = {}
        processed_profile = test_processed_profile()
        root_conan = self.retriever.root(str(content), processed_profile)
        deps_graph = self.builder.load_graph(root_conan, update, update, self.remotes,
                                             processed_profile)
        self.output.write("\n".join(self.resolver.output))
        return deps_graph

    def test_local_basic(self):
        for expr, solution in [(">0.0", "2.2.1"),
                               (">0.1,<1", "0.3"),
                               (">0.1,<1||2.1", "2.1"),
                               ("", "2.2.1"),
                               ("~0", "0.3"),
                               ("~=1", "1.2.1"),
                               ("~1.1", "1.1.2"),
                               ("~=2", "2.2.1"),
                               ("~=2.1", "2.1"),
                               ]:
            deps_graph = self.build_graph(TestConanFile("Hello", "1.2",
                                                        requires=["Say/[%s]@myuser/testing" % expr]))

            self.assertEqual(2, len(deps_graph.nodes))
            hello = _get_nodes(deps_graph, "Hello")[0]
            say = _get_nodes(deps_graph, "Say")[0]
            self.assertEqual(_get_edges(deps_graph), {Edge(hello, say)})

            self.assertEqual(hello.ref, None)
            conanfile = hello.conanfile
            self.assertEqual(conanfile.version, "1.2")
            self.assertEqual(conanfile.name, "Hello")
            say_ref = ConanFileReference.loads("Say/%s@myuser/testing" % solution)
            self.assertEqual(_clear_revs(conanfile.requires), Requirements(str(say_ref)))

    def test_remote_basic(self):
        self.resolver._local_search = None
        remote_packages = []
        for v in ["0.1", "0.2", "0.3", "1.1", "1.1.2", "1.2.1", "2.1", "2.2.1"]:
            say_ref = ConanFileReference.loads("Say/[%s]@myuser/testing" % v)
            remote_packages.append(say_ref)
        self.remotes.add("myremote", "myurl")
        self.remote_manager.packages = remote_packages
        for expr, solution in [(">0.0", "2.2.1"),
                               (">0.1,<1", "0.3"),
                               (">0.1,<1||2.1", "2.1"),
                               ("", "2.2.1"),
                               ("~0", "0.3"),
                               ("~=1", "1.2.1"),
                               ("~1.1", "1.1.2"),
                               ("~=2", "2.2.1"),
                               ("~=2.1", "2.1"),
                               ]:
            deps_graph = self.build_graph(TestConanFile("Hello", "1.2",
                                                        requires=["Say/[%s]@myuser/testing" % expr]),
                                          update=True)
            self.assertEqual(self.remote_manager.count, {'Say': 1})
            self.assertEqual(2, len(deps_graph.nodes))
            hello = _get_nodes(deps_graph, "Hello")[0]
            say = _get_nodes(deps_graph, "Say")[0]
            self.assertEqual(_get_edges(deps_graph), {Edge(hello, say)})

            self.assertEqual(hello.ref, None)
            conanfile = hello.conanfile
            self.assertEqual(conanfile.version, "1.2")
            self.assertEqual(conanfile.name, "Hello")
            say_ref = ConanFileReference.loads("Say/%s@myuser/testing" % solution)
            self.assertEqual(_clear_revs(conanfile.requires), Requirements(str(say_ref)))

    def test_remote_optimized(self):
        self.resolver._local_search = None
        remote_packages = []
        self.remotes.add("myremote", "myurl")
        for v in ["0.1", "0.2", "0.3", "1.1", "1.1.2", "1.2.1", "2.1", "2.2.1"]:
            say_ref = ConanFileReference.loads("Say/%s@myuser/testing" % v)
            remote_packages.append(say_ref)
        self.remote_manager.packages = remote_packages

        dep_content = """from conans import ConanFile
class Dep1Conan(ConanFile):
    requires = "Say/[%s]@myuser/testing"
"""
        dep_ref = ConanFileReference.loads("Dep1/0.1@myuser/testing")
        self.retriever.save_recipe(dep_ref, dep_content % ">=0.1")
        dep_ref = ConanFileReference.loads("Dep2/0.1@myuser/testing")
        self.retriever.save_recipe(dep_ref, dep_content % ">=0.1")

        hello_content = """from conans import ConanFile
class HelloConan(ConanFile):
    name = "Hello"
    requires = "Dep1/0.1@myuser/testing", "Dep2/0.1@myuser/testing"
"""
        deps_graph = self.build_graph(hello_content, update=True)
        self.assertEqual(4, len(deps_graph.nodes))
        hello = _get_nodes(deps_graph, "Hello")[0]
        say = _get_nodes(deps_graph, "Say")[0]
        dep1 = _get_nodes(deps_graph, "Dep1")[0]
        dep2 = _get_nodes(deps_graph, "Dep2")[0]
        self.assertEqual(_get_edges(deps_graph), {Edge(hello, dep1), Edge(hello, dep2),
                                                  Edge(dep1, say), Edge(dep2, say)})

        # Most important check: counter of calls to remote
        self.assertEqual(self.remote_manager.count, {'Say': 1})

    @parameterized.expand([("", "0.3", None, None),
                           ('"Say/1.1@myuser/testing"', "1.1", False, False),
                           ('"Say/0.2@myuser/testing"', "0.2", False, True),
                           ('("Say/1.1@myuser/testing", "override")', "1.1", True, False),
                           ('("Say/0.2@myuser/testing", "override")', "0.2", True, True),
                           # ranges
                           ('"Say/[<=1.2]@myuser/testing"', "1.2.1", False, False),
                           ('"Say/[>=0.2,<=1.0]@myuser/testing"', "0.3", False, True),
                           ('"Say/[>=0.2 <=1.0]@myuser/testing"', "0.3", False, True),
                           ('("Say/[<=1.2]@myuser/testing", "override")', "1.2.1", True, False),
                           ('("Say/[>=0.2,<=1.0]@myuser/testing", "override")', "0.3", True, True),
                           ('("Say/[>=0.2 <=1.0]@myuser/testing", "override")', "0.3", True, True),
                           ])
    def transitive_test(self, version_range, solution, override, valid):
        hello_text = TestConanFile("Hello", "1.2",
                                   requires=["Say/[>0.1, <1]@myuser/testing"])
        hello_ref = ConanFileReference.loads("Hello/1.2@myuser/testing")
        self.retriever.save_recipe(hello_ref, hello_text)

        chat_content = """
from conans import ConanFile

class ChatConan(ConanFile):
    name = "Chat"
    version = "2.3"
    requires = "Hello/1.2@myuser/testing", %s
"""
        if valid is False:
            with six.assertRaisesRegex(self, ConanException, "not valid"):
                self.build_graph(chat_content % version_range)
            return

        deps_graph = self.build_graph(chat_content % version_range)
        hello = _get_nodes(deps_graph, "Hello")[0]
        say = _get_nodes(deps_graph, "Say")[0]
        chat = _get_nodes(deps_graph, "Chat")[0]
        edges = {Edge(hello, say), Edge(chat, hello)}
        if override is not None:
            self.assertIn("overridden", self.output)
        else:
            self.assertNotIn("overridden", self.output)
        if override is False:
            edges = {Edge(hello, say), Edge(chat, say), Edge(chat, hello)}

        if valid is True:
            self.assertIn(" valid", self.output)
            self.assertNotIn("not valid", self.output)
        elif valid is False:
            self.assertIn("not valid", self.output)
        self.assertEqual(3, len(deps_graph.nodes))

        self.assertEqual(_get_edges(deps_graph), edges)

        self.assertEqual(hello.ref.copy_clear_rev(), hello_ref)
        conanfile = hello.conanfile
        self.assertEqual(conanfile.version, "1.2")
        self.assertEqual(conanfile.name, "Hello")
        say_ref = ConanFileReference.loads("Say/%s@myuser/testing" % solution)
        self.assertEqual(_clear_revs(conanfile.requires), Requirements(str(say_ref)))

    def duplicated_error_test(self):
        content = TestConanFile("log4cpp", "1.1.1")
        log4cpp_ref = ConanFileReference.loads("log4cpp/1.1.1@myuser/testing")
        self.retriever.save_recipe(log4cpp_ref, content)

        content = """
from conans import ConanFile

class LoggerInterfaceConan(ConanFile):
    name = "LoggerInterface"
    version = "0.1.1"

    def requirements(self):
        self.requires("log4cpp/[~1.1]@myuser/testing")
"""
        logiface_ref = ConanFileReference.loads("LoggerInterface/0.1.1@myuser/testing")
        self.retriever.save_recipe(logiface_ref, content)

        content = """
from conans import ConanFile

class OtherConan(ConanFile):
    name = "other"
    version = "2.0.11549"
    requires = "LoggerInterface/[~0.1]@myuser/testing"
"""
        other_ref = ConanFileReference.loads("other/2.0.11549@myuser/testing")
        self.retriever.save_recipe(other_ref, content)

        content = """
from conans import ConanFile

class Project(ConanFile):
    requires = "LoggerInterface/[~0.1]@myuser/testing", "other/[~2.0]@myuser/testing"
"""
        deps_graph = self.build_graph(content)

        log4cpp = _get_nodes(deps_graph, "log4cpp")[0]
        logger_interface = _get_nodes(deps_graph, "LoggerInterface")[0]
        other = _get_nodes(deps_graph, "other")[0]

        self.assertEqual(4, len(deps_graph.nodes))

        self.assertEqual(log4cpp.ref.copy_clear_rev(), log4cpp_ref)
        conanfile = log4cpp.conanfile
        self.assertEqual(conanfile.version, "1.1.1")
        self.assertEqual(conanfile.name, "log4cpp")

        self.assertEqual(logger_interface.ref.copy_clear_rev(), logiface_ref)
        conanfile = logger_interface.conanfile
        self.assertEqual(conanfile.version, "0.1.1")
        self.assertEqual(conanfile.name, "LoggerInterface")

        self.assertEqual(other.ref.copy_clear_rev(), other_ref)
        conanfile = other.conanfile
        self.assertEqual(conanfile.version, "2.0.11549")
        self.assertEqual(conanfile.name, "other")

    def different_user_channel_resolved_correctly_test(self):
        server1 = TestServer()
        server2 = TestServer()
        servers = OrderedDict([("server1", server1), ("server2", server2)])

        client = TurboTestClient(servers=servers)
        ref1 = ConanFileReference.loads("lib/1.0@conan/stable")
        ref2 = ConanFileReference.loads("lib/1.0@conan/testing")

        client.create(ref1, conanfile=GenConanfile())
        client.upload_all(ref1, remote="server1")

        client.create(ref2, conanfile=GenConanfile())
        client.upload_all(ref2, remote="server2")

        client2 = TurboTestClient(servers=servers)
        client2.run("install lib/[>=1.0]@conan/testing")
        self.assertIn("lib/1.0@conan/testing: Retrieving package {} "
                      "from remote 'server2' ".format(NO_SETTINGS_PACKAGE_ID), client2.out)



import unittest

from conans.test.utils.tools import TestClient


class TransitiveOptionsAffectPackageIDTest(unittest.TestCase):

    def basic_test(self):
        client = TestClient()
        conanfile = '''from conans import ConanFile
class Pkg(ConanFile):
    options = {"shared": [True, False], "num": [1, 2, 3]}
    default_options= "shared=False", "num=1"
'''

        client.save({"conanfile.py": conanfile})
        client.run("create . PkgA/0.1@user/testing")
        self.assertIn("e522a4e1c101d2a2b4f0009497ae0d0b6a29ae65", client.out)
        client.run("create . PkgA/0.1@user/testing -o PkgA:shared=True -o PkgA:num=2")
        self.assertIn("18d072f161b7c8d77083ca6dec0c5222f4bacf22", client.out)
        client.save({"conanfile.py": conanfile + "    requires = 'PkgA/0.1@user/testing'"})
        client.run("create . PkgB/0.1@user/testing")
        client.save({"conanfile.py": conanfile + "    requires = 'PkgB/0.1@user/testing'"})
        client.run("create . PkgC/0.1@user/testing")
        client.save({"conanfile.py": conanfile + "    requires = 'PkgC/0.1@user/testing'"})

        client.run("install .")
        self.assertIn("PkgA/0.1@user/testing:e522a4e1c101d2a2b4f0009497ae0d0b6a29ae65", client.out)
        self.assertIn("PkgB/0.1@user/testing:fba7313915d1eaaa52c0a64d2f576c62e2cabd5d", client.out)
        self.assertIn("PkgC/0.1@user/testing:95cf05dd9309c4c4f3c5d8a881ff13d93481974e", client.out)
        client.run("install . -o PkgA:shared=True -o PkgA:num=2")
        # Only PkgA changes!
        self.assertIn("PkgA/0.1@user/testing:18d072f161b7c8d77083ca6dec0c5222f4bacf22", client.out)
        self.assertIn("PkgB/0.1@user/testing:fba7313915d1eaaa52c0a64d2f576c62e2cabd5d", client.out)
        self.assertIn("PkgC/0.1@user/testing:95cf05dd9309c4c4f3c5d8a881ff13d93481974e", client.out)



import unittest

from bottle import request

from conans.test.utils.tools import TestClient, TestServer
from conans.util.env_reader import get_env

conanfile = """
from conans import ConanFile

class OpenSSLConan(ConanFile):
    name = "Hello"
    version = "0.1"
"""


class AuthorizationHeaderSpy(object):
    ''' Generic plugin to handle Authorization header. Must be extended and implement
    some abstract methods in subclasses'''

    name = 'authorizationheaderspy'
    api = 2

    def __init__(self):
        self.auths = []

    def apply(self, callback, context):  # @UnusedVariable
        auth = request.headers.get("Authorization")
        name = callback.__name__
        self.auths.append((name, auth))
        return callback


class ReturnHandlerPlugin(object):

    name = 'ReturnHandlerPluginSpy'
    api = 2

    def apply(self, callback, _):
        '''Apply plugin'''
        def wrapper(*args, **kwargs):
            '''Capture possible exceptions to manage the return'''
            result = callback(*args, **kwargs)
            if isinstance(result, dict):
                for k, v in result.items():
                    result[k] = v.split("?signature=")[0]
            return result
        return wrapper


class AuthorizeBearerTest(unittest.TestCase):

    def basic_test(self):
        auth = AuthorizationHeaderSpy()
        server = TestServer(plugins=[auth])
        servers = {"default": server}
        client = TestClient(servers=servers, users={"default": [("lasote", "mypass")]})
        client.save({"conanfile.py": conanfile})
        client.run("export . lasote/stable")
        errors = client.run("upload Hello/0.1@lasote/stable")
        self.assertFalse(errors)

        if not client.cache.config.revisions_enabled:
            expected_calls = [('ping', None),
                              ('get_recipe_manifest_url', None),
                              ('check_credentials', None),
                              ('authenticate', 'Basic'),
                              ('get_recipe_snapshot', 'Bearer'),
                              ('get_conanfile_upload_urls', 'Bearer'),
                              ('put', None)]
        else:
            expected_calls = [('ping', None),
                              ('get_recipe_file', None),
                              ('check_credentials', None),
                              ('authenticate', 'Basic'),
                              ('get_recipe_file_list', 'Bearer'),
                              ('upload_recipe_file', 'Bearer')]

        self.assertEqual(len(expected_calls), len(auth.auths))
        for i, (method, auth_type) in enumerate(expected_calls):
            real_call = auth.auths[i]
            self.assertEqual(method, real_call[0])
            if auth_type:
                self.assertIn(auth_type, real_call[1])

    @unittest.skipIf(get_env("TESTING_REVISIONS_ENABLED", False), "ApiV1 test")
    def no_signature_test(self):
        auth = AuthorizationHeaderSpy()
        retur = ReturnHandlerPlugin()
        server = TestServer(plugins=[auth, retur])
        servers = {"default": server}
        client = TestClient(servers=servers, users={"default": [("lasote", "mypass")]})
        client.save({"conanfile.py": conanfile})
        client.run("export . lasote/stable")
        # Upload will fail, as conan_server is expecting a signed URL
        errors = client.run("upload Hello/0.1@lasote/stable", assert_error=True)
        self.assertTrue(errors)

        expected_calls = [('ping', None),
                          ('get_recipe_manifest_url', None),
                          ('check_credentials', None),
                          ('authenticate', 'Basic'),
                          ('get_recipe_snapshot', 'Bearer'),
                          ('get_conanfile_upload_urls', 'Bearer'),
                          ('put', 'Bearer')]

        self.assertEqual(len(expected_calls), len(auth.auths))
        for i, (method, auth_type) in enumerate(expected_calls):
            real_call = auth.auths[i]
            self.assertEqual(method, real_call[0])
            if auth_type:
                self.assertIn(auth_type, real_call[1])

        # The Bearer of the last two calls must be identical
        self.assertEqual(auth.auths[-1][1], auth.auths[-2][1])



import os
import unittest

from conans.client import tools
from conans.client.cache.cache import PROFILES_FOLDER
from conans.paths import CONANFILE
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient
from conans.util.files import save


class DefaultProfileTest(unittest.TestCase):

    def conanfile_txt_incomplete_profile_test(self):
        conanfile = '''from conans import ConanFile
class MyConanfile(ConanFile):
    pass
'''

        client = TestClient()
        save(client.cache.default_profile_path, "[env]\nValue1=A")

        client.save({CONANFILE: conanfile})
        client.run("create . Pkg/0.1@lasote/stable")
        self.assertIn("Pkg/0.1@lasote/stable: Package '%s' created" % NO_SETTINGS_PACKAGE_ID,
                      client.out)

        client.save({"conanfile.txt": "[requires]\nPkg/0.1@lasote/stable"}, clean_first=True)
        client.run('install .')
        self.assertIn("Pkg/0.1@lasote/stable: Already installed!", client.out)

    def change_default_profile_test(self):
        br = '''
import os
from conans import ConanFile

class MyConanfile(ConanFile):
    name = "mylib"
    version = "0.1"

    def build(self):
        assert(os.environ.get("Value1") == "A")

'''
        tmp = temp_folder()
        default_profile_path = os.path.join(tmp, "myprofile")
        save(default_profile_path, "[env]\nValue1=A")
        client = TestClient()
        client.run("config set general.default_profile='%s'" % default_profile_path)
        client.save({CONANFILE: br})
        client.run("export . lasote/stable")
        client.run('install mylib/0.1@lasote/stable --build')

        # Now use a name, in the default profile folder
        os.unlink(default_profile_path)
        save(os.path.join(client.cache.profiles_path, "other"), "[env]\nValue1=A")
        client.run("config set general.default_profile=other")
        client.save({CONANFILE: br})
        client.run("export . lasote/stable")
        client.run('install mylib/0.1@lasote/stable --build')

    def test_profile_applied_ok(self):
        br = '''
import os
from conans import ConanFile

class BuildRequireConanfile(ConanFile):
    name = "br"
    version = "1.0"
    settings = "os", "compiler", "arch"

    def package_info(self):
        self.env_info.MyVAR="from_build_require"
'''

        client = TestClient()

        default_profile = """
[settings]
os=Windows
compiler=Visual Studio
compiler.version=14
compiler.runtime=MD
arch=x86

[options]
mypackage:option1=2

[build_requires]
br/1.0@lasote/stable
"""
        save(client.cache.default_profile_path, default_profile)

        client.save({CONANFILE: br})
        client.run("export . lasote/stable")

        cf = '''
import os
from conans import ConanFile

class MyConanfile(ConanFile):
    name = "mypackage"
    version = "0.1"
    settings = "os", "compiler", "arch"
    options = {"option1": ["1", "2"]}
    default_options = "option1=1"

    def configure(self):
        assert(self.settings.os=="Windows")
        assert(self.settings.compiler=="Visual Studio")
        assert(self.settings.compiler.version=="14")
        assert(self.settings.compiler.runtime=="MD")
        assert(self.settings.arch=="x86")
        assert(self.options.option1=="2")
    
    def build(self):
        # This has changed, the value from profile higher priority than build require
        assert(os.environ["MyVAR"]=="%s")

        '''

        # First apply just the default profile, we should get the env MYVAR="from_build_require"
        client.save({CONANFILE: cf % "from_build_require"}, clean_first=True)
        client.run("export . lasote/stable")
        client.run('install mypackage/0.1@lasote/stable --build missing')

        # Then declare in the default profile the var, it should be prioritized from the br
        default_profile_2 = default_profile + "\n[env]\nMyVAR=23"
        save(client.cache.default_profile_path, default_profile_2)
        client.save({CONANFILE: cf % "23"}, clean_first=True)
        client.run("export . lasote/stable")
        client.run('install mypackage/0.1@lasote/stable --build missing')

    def test_env_default_profile(self):
        conanfile = '''
import os
from conans import ConanFile

class MyConanfile(ConanFile):

    def build(self):
        self.output.info(">>> env_variable={}".format(os.environ.get('env_variable'))) 
'''

        client = TestClient()
        client.save({CONANFILE: conanfile})

        # Test with the 'default' profile
        env_variable = "env_variable=profile_default"
        save(client.cache.default_profile_path, "[env]\n" + env_variable)
        client.run("create . name/version@user/channel")
        self.assertIn(">>> " + env_variable, client.out)

        # Test with a profile set using and environment variable
        tmp = temp_folder()
        env_variable = "env_variable=profile_environment"
        default_profile_path = os.path.join(tmp, 'env_profile')
        save(default_profile_path, "[env]\n" + env_variable)
        with tools.environment_append({'CONAN_DEFAULT_PROFILE_PATH': default_profile_path}):
            client.run("create . name/version@user/channel")
            self.assertIn(">>> " + env_variable, client.out)

        # Use relative path defined in environment variable
        env_variable = "env_variable=relative_profile"
        rel_path = os.path.join('..', 'env_rel_profile')
        self.assertFalse(os.path.isabs(rel_path))
        default_profile_path = os.path.join(client.cache_folder,
                                            PROFILES_FOLDER, rel_path)
        save(default_profile_path, "[env]\n" + env_variable)
        with tools.environment_append({'CONAN_DEFAULT_PROFILE_PATH': rel_path}):
            client.run("create . name/version@user/channel")
            self.assertIn(">>> " + env_variable, client.out)

        # Use non existing path
        profile_path = os.path.join(tmp, "this", "is", "a", "path")
        self.assertTrue(os.path.isabs(profile_path))
        with tools.environment_append({'CONAN_DEFAULT_PROFILE_PATH': profile_path}):
            client.run("create . name/version@user/channel", assert_error=True)
            self.assertIn("Environment variable 'CONAN_DEFAULT_PROFILE_PATH' must point to "
                          "an existing profile file.", client.out)





import os
import unittest

from parameterized import parameterized

from conans.test.utils.tools import TestClient
from conans.util.files import load, mkdir


class CMakeFoldersTest(unittest.TestCase):

    @parameterized.expand([(True, True), (False, True), (True, False), (False, False)])
    def basic_test(self, no_copy_source, cmake_install):
        client = TestClient()
        if not cmake_install:
            package_code = """
    def package(self):
        self.copy("*.h", src="src", dst="include")
"""
        else:
            package_code = "cmake.install()"

        conanfile = """from conans import ConanFile, CMake, load
import os
class Conan(ConanFile):
    settings = "os", "compiler", "arch", "build_type"
    exports_sources = "src/*"
    no_copy_source = {}
    def build(self):
        cmake = CMake(self)
        cmake.configure(source_folder="src",
                        cache_build_folder="build")
        cmake.build()
        {}

    def package_info(self):
        self.output.info("HEADER %s" % load(os.path.join(self.package_folder, "include/header.h")))
    """.format(no_copy_source, package_code)
        cmake = """set(CMAKE_CXX_COMPILER_WORKS 1)
project(Chat NONE)
cmake_minimum_required(VERSION 2.8.12)
file(READ header.h MYCONTENTS)
message(STATUS "HEADER CMAKE CONTENTS ${MYCONTENTS}")
install(FILES header.h DESTINATION include)
"""
        client.save({"conanfile.py": conanfile,
                     "src/CMakeLists.txt": cmake,
                     "src/header.h": "//myheader.h"})
        client.run("create . Hello/0.1@lasote/channel")
        self.assertIn("Hello/0.1@lasote/channel: HEADER //myheader.h", client.out)
        self.assertIn("-- HEADER CMAKE CONTENTS //myheader.h", client.out)
        # Now local flow
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(build_folder)
        client.current_folder = build_folder
        client.run("install ..")
        client.run("build ..")  # same as --build-folder=. --source-folder=..
        self.assertIn("-- HEADER CMAKE CONTENTS //myheader.h", client.out)
        if not cmake_install:
            client.run("package ..")  # same as --build-folder=. --source-folder=..
        self.assertTrue(os.path.exists(os.path.join(build_folder, "conaninfo.txt")))
        self.assertTrue(os.path.exists(os.path.join(build_folder, "conanbuildinfo.txt")))
        self.assertEqual(load(os.path.join(build_folder, "package/include/header.h")), "//myheader.h")
